import unittest.mock

import pytest

import smartcard.Observer


def test_state_changes():
    """Verify that the "changed" state can be set and cleared."""

    # Base state
    observable = smartcard.Observer.Observable()
    assert not observable.hasChanged(), "Observable must not start in a changed state"

    # Set
    observable.setChanged()
    assert observable.hasChanged(), ".setChanged() must *set* the state"
    observable.setChanged()
    assert observable.hasChanged(), ".setChanged() must not *toggle* the state"

    # Clear
    observable.clearChanged()
    assert not observable.hasChanged(), ".clearChanged() must *clear* the state"
    observable.clearChanged()
    assert not observable.hasChanged(), ".clearChanged() must not *toggle* the state"


@pytest.mark.parametrize(
    "unbound_method",
    (
        # Observer management methods
        smartcard.Observer.Observable.addObserver,
        smartcard.Observer.Observable.countObservers,
        smartcard.Observer.Observable.deleteObserver,
        smartcard.Observer.Observable.deleteObservers,
        smartcard.Observer.Observable.notifyObservers,
        # State management methods
        smartcard.Observer.Observable.clearChanged,
        smartcard.Observer.Observable.hasChanged,
        smartcard.Observer.Observable.setChanged,
    ),
)
def test_synchronization(unbound_method):
    """Verify that all Observable methods are synchronized.

    The `.mutex` attribute is mocked and is confirmed to be acquired and released.

    The test implementation here doesn't necessarily make valid method calls;
    it relies on the knowledge that most Observable methods are decorated
    and that the lock is acquired before the underlying method is called
    (and thus, before Python realizes that required method parameters are unfilled
    and raises a TypeError).
    """

    mutex = unittest.mock.MagicMock()
    # Ensure the context manager protocol is functional.
    mutex.__enter__.return_value = mutex

    observable = smartcard.Observer.Observable()
    observable.mutex = mutex

    try:
        # Call the unbound Observable method, passing *observable* as `self`.
        unbound_method(observable)  # noqa
    except TypeError:
        # Ignore TypeErrors caused by missing mandatory method arguments.
        pass

    mutex.__enter__.assert_called_once()
    mutex.__exit__.assert_called_once()


def test_registered_observers_are_always_notified():
    """Verify all observers are notified, even if the observer list changes."""

    class JealousObserver(smartcard.Observer.Observer):
        update_count = 0

        def update(self, observable, _):
            # Try to prevent all other observers from receiving an update.
            for other_observer in observers - {self}:
                try:
                    observable.deleteObserver(other_observer)
                except ValueError:
                    pass

            # Track the number of times an update was received.
            JealousObserver.update_count += 1

    jealous_observer_count = 10
    observers = {JealousObserver() for _ in range(jealous_observer_count)}

    # Create a fair observable and notify the observers of a change.
    fair_observable = smartcard.Observer.Observable()
    [fair_observable.addObserver(observer) for observer in observers]
    fair_observable.setChanged()
    fair_observable.notifyObservers()

    # Confirm that the observers have all erased each other from the list of observers.
    assert fair_observable.countObservers() == 0

    # Confirm that the observers were all updated, despite their jealous efforts.
    assert JealousObserver.update_count == jealous_observer_count


def test_double_observer_additions():
    """Verify that an observer cannot be added twice."""

    observer = smartcard.Observer.Observer()
    observable = smartcard.Observer.Observable()
    observable.addObserver(observer)
    observable.addObserver(observer)

    assert observable.countObservers() == 1


def test_double_observer_removals():
    """Verify that an observer cannot be removed twice."""

    observer = smartcard.Observer.Observer()
    observable = smartcard.Observer.Observable()
    observable.addObserver(observer)
    observable.deleteObserver(observer)

    with pytest.raises(ValueError, match="x not in list"):
        observable.deleteObserver(observer)


def test_no_notifications_when_no_changes():
    """Verify that no notifications are sent when no changes are detected."""

    class Observer(smartcard.Observer.Observer):
        def update(self, *_, **__):
            raise RuntimeError("no updates were expected")

    observable = smartcard.Observer.Observable()
    observable.addObserver(Observer())
    # Attempt to send a notification when no changes have been made.
    # Nothing needs to be asserted here;
    # the lack of a RuntimeError during test execution demonstrates the test passed.
    observable.notifyObservers()


def test_default_observer_updates():
    """Call the default Observer.update() method.

    This test exists solely for the purposes of reaching 100% test coverage.
    It can be removed if Observer becomes an abstract base class.
    """

    observable = smartcard.Observer.Observable()
    assert smartcard.Observer.Observer().update(observable, "arg") is None



from smartcard.guid import GUIDToStr, strToGUID


def test_roundtrip_string():
    string = "{AD4F1667-EA75-4124-84D4-641B3B197C65}"
    assert GUIDToStr(strToGUID(string)) == string


def test_roundtrip_list_of_ints():
    list_of_ints = [
        103,
        22,
        79,
        173,
        117,
        234,
        36,
        65,
        132,
        212,
        100,
        27,
        59,
        25,
        124,
        101,
    ]
    assert strToGUID(GUIDToStr(list_of_ints)) == list_of_ints



"""The CardConnectionEvent is sent to CardConnectionObserver objects
when a CardConnection event occurs.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


class CardConnectionEvent:
    """Base class for card connection events.

    This event is notified by CardConnection objects."""

    def __init__(self, type, args=None):
        """
        @param type:   'connect', 'reconnect', 'disconnect', 'command', 'response'
        @param args:   None for 'connect', 'reconnect' or 'disconnect'
                command APDU byte list for 'command'
                [response data, sw1, sw2] for 'response'
        """
        self.type = type
        self.args = args



#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: List card interfaces

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import platform
import sys

import smartcard.guid
from smartcard.scard import *

if "winscard" == resourceManager:

    znewcardName = "dummy-card"
    znewcardATR = [
        0x3B,
        0x77,
        0x94,
        0x00,
        0x00,
        0x82,
        0x30,
        0x00,
        0x13,
        0x6C,
        0x9F,
        0x22,
    ]
    znewcardMask = [
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
    ]
    znewcardPrimGuid = smartcard.guid.strToGUID(
        "{128F3806-4F70-4ccf-977A-60C390664840}"
    )
    znewcardSecGuid = smartcard.guid.strToGUID("{EB7F69EA-BA20-47d0-8C50-11CFDEB63BBE}")

    def main():
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise scard.error(
                "Failed to establish context: " + SCardGetErrorMessage(hresult)
            )
        print("Context established!")

        try:

            # list interfaces for a known card
            expectedCard = "Schlumberger Cryptoflex 8k v2"
            hresult, interfaces = SCardListInterfaces(hcontext, expectedCard)
            if hresult != SCARD_S_SUCCESS:
                raise scard.error(
                    "Failed to list interfaces: " + SCardGetErrorMessage(hresult)
                )
            print("Interfaces for ", expectedCard, ":", interfaces)

            # introduce a card (forget first in case it is already present)
            hresult = SCardForgetCardType(hcontext, znewcardName)
            print("Introducing card " + znewcardName)
            hresult = SCardIntroduceCardType(
                hcontext,
                znewcardName,
                znewcardPrimGuid,
                znewcardPrimGuid + znewcardSecGuid,
                znewcardATR,
                znewcardMask,
            )
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    "Failed to introduce card type: " + SCardGetErrorMessage(hresult)
                )

            # list card interfaces
            hresult, interfaces = SCardListInterfaces(hcontext, znewcardName)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    "Failed to list interfaces: " + SCardGetErrorMessage(hresult)
                )
            for i in interfaces:
                print(
                    "Interface for " + znewcardName + " :", smartcard.guid.GUIDToStr(i)
                )

            print("Forgetting card " + znewcardName)
            hresult = SCardForgetCardType(hcontext, znewcardName)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    "Failed to remove card type: " + SCardGetErrorMessage(hresult)
                )

        finally:
            hresult2 = SCardReleaseContext(hcontext)
            if hresult2 != SCARD_S_SUCCESS:
                raise error(
                    "Failed to release context: " + SCardGetErrorMessage(hresult)
                )
            print("Released context.")

    main()

elif "pcsclite" == resourceManager:
    print("SCardListInterfaces not supported by pcsc lite")


if "win32" == sys.platform:
    print("press Enter to continue")
    sys.stdin.read(1)



#! /usr/bin/env python3
"""Unit tests for smartcard.ExclusiveTransmitCardConnection.

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import random
import threading
import time
import unittest

# from smartcard.CardConnectionObserver import ConsoleCardConnectionObserver
from smartcard.CardRequest import CardRequest
from smartcard.CardType import AnyCardType
from smartcard.ExclusiveTransmitCardConnection import ExclusiveTransmitCardConnection

# define the apdus used in this script
GET_RESPONSE = [0xA0, 0xC0, 00, 00]
SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]


def signalEvent(evt):
    """A simple callback that signals an event."""
    evt.set()


class testthread(threading.Thread):
    """A test thread that repetitevely sends APDUs to a card within a
    transaction."""

    def __init__(self, threadindex):
        """Connect to a card with an ExclusiveTransmitCardConnection."""
        threading.Thread.__init__(self)

        self.threadindex = threadindex

        # request any card type
        cardtype = AnyCardType()
        cardrequest = CardRequest(timeout=5, cardType=cardtype)
        cardservice = cardrequest.waitforcard()

        # attach our decorator
        cardservice.connection = ExclusiveTransmitCardConnection(cardservice.connection)

        # uncomment to attach the console tracer
        # observer=ConsoleCardConnectionObserver()
        # cardservice.connection.addObserver(observer)

        # connect to the card
        cardservice.connection.connect()

        self.cardservice = cardservice

        # this event will signal the end of the thread
        self.evtStop = threading.Event()

        # this timer will set the event stop event in 30s
        timer = threading.Timer(10, signalEvent, [self.evtStop])
        timer.start()
        self.countTransmitted = 0

    def run(self):
        """Transmit APDUS with a random interval to the card."""
        connection = self.cardservice.connection
        while not self.evtStop.is_set():
            try:
                connection.lock()

                apdu = SELECT + DF_TELECOM
                _, sw1, sw2 = connection.transmit(apdu)

                if 0x90 == (sw1 & 0xF0):
                    apdu = GET_RESPONSE + [sw2]
                    _, sw1, sw2 = connection.transmit(apdu)
            finally:
                connection.unlock()
            self.countTransmitted = self.countTransmitted + 1
            time.sleep(float(random.uniform(1, 3)) * 0.01)


class testcase_cardmonitor(unittest.TestCase):
    """Test smartcard framework card monitoring classes"""

    def testcase_cardmonitorthread(self):
        """card monitor thread"""
        threads = []
        for i in range(0, 4):
            t = testthread(i)
            threads.append(t)
        for t in threads:
            t.start()
        for t in threads:
            t.join()
        # for t in threads:
        #     print(f"Thread {t.threadindex}: transmitted {t.countTransmitted} apdus.")


if __name__ == "__main__":
    unittest.main(verbosity=1)



#! /usr/bin/env python3
"""Unit tests for smartcard.CardType

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
import unittest

sys.path += [".."]

try:
    from local_config import expectedATRinReader
except ImportError:
    print("execute test suite first to generate the local_config.py file")
    sys.exit()


from smartcard.CardType import AnyCardType, ATRCardType
from smartcard.System import readers


class testcase_CardType(unittest.TestCase):
    """Test case for CardType."""

    def testcase_AnyCardType(self):
        """Test smartcard.AnyCardType."""

        ct = AnyCardType()
        for reader in readers():
            if expectedATRinReader[reader.name]:
                connection = reader.createConnection()
                connection.connect()
                self.assertEqual(True, ct.matches(connection.getATR()))
                self.assertEqual(True, ct.matches(connection.getATR(), reader))
                connection.disconnect()
                connection.release()

    def testcase_ATRCardTypeWithoutMask(self):
        """Test smartcard.ATRCardType without mask."""

        for reader in readers():
            if expectedATRinReader[reader.name]:
                ct = ATRCardType(expectedATRinReader[reader.name])
                connection = reader.createConnection()
                connection.connect()
                self.assertEqual(True, ct.matches(connection.getATR()))
                self.assertEqual(True, ct.matches(connection.getATR(), reader))
                connection.disconnect()
                connection.release()

    def testcase_ATRCardTypeMisMatchWithoutMask(self):
        """Test smartcard.ATRCardType mismatch without mask."""

        for reader in readers():
            if expectedATRinReader[reader.name]:
                atr = list(expectedATRinReader[reader.name])
                # change the last byte of the expected atr
                atr[-1] = 0xFF
                ct = ATRCardType(atr)
                connection = reader.createConnection()
                connection.connect()
                self.assertEqual(False, ct.matches(connection.getATR()))
                self.assertEqual(False, ct.matches(connection.getATR(), reader))
                connection.disconnect()
                connection.release()

    def testcase_ATRCardTypeWithMask(self):
        """Test smartcard.ATRCardType with mask."""

        for reader in readers():
            if expectedATRinReader[reader.name]:
                mask = [0xFF for x in expectedATRinReader[reader.name]]
                # don't look to the last byte
                mask[-1] = 0x00
                ct = ATRCardType(expectedATRinReader[reader.name], mask)
                connection = reader.createConnection()
                connection.connect()
                atr = connection.getATR()
                connection.disconnect()
                connection.release()
                # change a bit in the last byte
                atr[-1] = atr[-1] ^ 0xFF
                self.assertEqual(True, ct.matches(atr))
                self.assertEqual(True, ct.matches(atr, reader))

    def testcase_ATRCardTypeWithMaskMismatch(self):
        """Test smartcard.ATRCardType with mask and mismatch."""

        for reader in readers():
            if expectedATRinReader[reader.name]:
                mask = [0xFF for x in expectedATRinReader[reader.name]]
                # don't look to the last byte
                mask[0] = mask[-1] = 0x00
                ct = ATRCardType(expectedATRinReader[reader.name], mask)
                connection = reader.createConnection()
                connection.connect()
                atr = connection.getATR()
                connection.disconnect()
                connection.release()
                # change a bit in the :-2 byte
                atr[-2] = atr[-2] ^ 0xFF
                self.assertEqual(False, ct.matches(atr))
                self.assertEqual(False, ct.matches(atr, reader))


if __name__ == "__main__":
    unittest.main(verbosity=1)



from smartcard.scard import *


def test_low_level():
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    assert hresult in (SCARD_S_SUCCESS, SCARD_E_NO_SERVICE)

    if hresult == SCARD_E_NO_SERVICE:
        return

    hresult, _ = SCardListReaders(hcontext, [])
    assert hresult in (SCARD_S_SUCCESS, SCARD_E_NO_READERS_AVAILABLE)

    # the computer we are using may not have a reader connected
    # so we can't do much

    hresult = SCardReleaseContext(hcontext)
    assert hresult == SCARD_S_SUCCESS



#! /usr/bin/env python3
"""
Example wxPython application that displays readers and inserted cards ATRs.
This example displays a snapshot of the readers and cards, there is no
automatic refresh of the readers and cards.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import smartcard.Exceptions
import smartcard.System
import smartcard.util

# wxPython GUI modules (https://www.wxpython.org/)
try:
    import wx
except ImportError:
    print(
        "You need wxpython (https://www.wxpython.org/) "
        + "to run this sample from the source code!"
    )
    print("press a key to continue")
    import msvcrt

    msvcrt.getch()
    import sys

    sys.exit()


def getATR(reader):
    """Return the ATR of the card inserted into the reader."""
    connection = reader.createConnection()
    atr = ""
    try:
        connection.connect()
        atr = smartcard.util.toHexString(connection.getATR())
        connection.disconnect()
    except smartcard.Exceptions.NoCardException:
        atr = "no card inserted"
    return atr


class pcscdiag(wx.Frame):

    def __init__(self, parent, title):
        wx.Frame.__init__(self, parent, -1, title, size=(600, 400))
        w, h = self.GetClientSize()
        self.tree = wx.TreeCtrl(
            self,
            wx.NewIdRef(),
            wx.DefaultPosition,
            (w, h),
            wx.TR_HAS_BUTTONS | wx.TR_EDIT_LABELS,
        )
        self.InitTree()
        self.OnExpandAll()

    def InitTree(self):
        self.tree.AddRoot("Readers and ReaderGroups")

        readerNode = self.tree.AppendItem(self.tree.GetRootItem(), "Readers")
        for reader in smartcard.System.readers():
            childReader = self.tree.AppendItem(readerNode, repr(reader))
            childCard = self.tree.AppendItem(childReader, getATR(reader))

        readerGroupNode = self.tree.AppendItem(
            self.tree.GetRootItem(), "Readers Groups"
        )
        for readergroup in smartcard.System.readergroups():
            childReaderGroup = self.tree.AppendItem(readerGroupNode, readergroup)
            readers = smartcard.System.readers(readergroup)
            for reader in readers:
                child = self.tree.AppendItem(childReaderGroup, repr(reader))

    def OnExpandAll(self):
        """expand all nodes"""
        root = self.tree.GetRootItem()
        fn = self.tree.Expand
        self.traverse(root, fn)
        self.tree.Expand(root)

    def traverse(self, traverseroot, function, cookie=0):
        """recursively walk tree control"""
        if self.tree.ItemHasChildren(traverseroot):
            firstchild, cookie = self.tree.GetFirstChild(traverseroot)
            function(firstchild)
            self.traverse(firstchild, function, cookie)

        child = self.tree.GetNextSibling(traverseroot)
        if child:
            function(child)
            self.traverse(child, function, cookie)


if __name__ == "__main__":
    app = wx.App()
    frame = pcscdiag(None, "Smartcard readers and reader groups")
    frame.Show()
    app.MainLoop()



#! /usr/bin/env python3
"""Unit tests for smartcard.pcsc.readers

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
import unittest

sys.path += [".."]

try:
    from local_config import (
        expectedATRinReader,
        expectedATRs,
        expectedReaderGroups,
        expectedReaders,
    )
except ImportError:
    print("execute test suite first to generate the local_config.py file")
    sys.exit()


from smartcard.pcsc.PCSCReader import PCSCReader
from smartcard.pcsc.PCSCReaderGroups import PCSCReaderGroups
from smartcard.scard import resourceManager

if "winscard" == resourceManager:

    class testcase_readergroups(unittest.TestCase):
        """Test smartcard framework readers factory methods"""

        def setUp(self):
            groups = PCSCReaderGroups().instance
            groups.remove("Pinpad$Readers")
            groups.remove("Biometric$Readers")

        def testcase_add(self):
            """Test for groups=groups+newgroups"""
            groupssnapshot = list(PCSCReaderGroups().instance)
            groups = PCSCReaderGroups().instance
            newgroup = "Pinpad$Readers"
            groups = groups + newgroup
            self.assertEqual(groups, groupssnapshot + [newgroup])
            groups.remove(newgroup)

        def testcase_addlist(self):
            """Test for groups=groups+[newgroups]"""
            groupssnapshot = list(PCSCReaderGroups().instance)
            groups = PCSCReaderGroups().instance
            newgroups = ["Pinpad$Readers", "Biometric$Readers"]
            groups = groups + newgroups
            self.assertEqual(groups, groupssnapshot + newgroups)
            for group in newgroups:
                groups.remove(group)

        def testcase_iadd(self):
            """Test for groups+=newgroup"""
            groupssnapshot = list(PCSCReaderGroups().instance)
            groups = PCSCReaderGroups().instance
            newgroup = "Pinpad$Readers"
            groups += newgroup
            self.assertEqual(groups, groupssnapshot + [newgroup])
            groups.remove(newgroup)

        def testcase_iaddlist(self):
            """Test for groups+=[newgroups]"""
            groupssnapshot = list(PCSCReaderGroups().instance)
            groups = PCSCReaderGroups().instance
            newgroups = ["Pinpad$Readers", "Biometric$Readers"]
            groups += newgroups
            self.assertEqual(groups, groupssnapshot + newgroups)
            for group in newgroups:
                groups.remove(group)

        def testcase_append(self):
            """Test for groups.append(newgroup)"""
            groupssnapshot = list(PCSCReaderGroups().instance)
            groups = PCSCReaderGroups().instance
            newgroup = "Pinpad$Readers"
            groups.append(newgroup)
            self.assertEqual(groups, groupssnapshot + [newgroup])
            groups.remove(newgroup)

        def testcase_insert(self):
            """Test for groups.insert(newgroup)"""
            groupssnapshot = list(PCSCReaderGroups().instance)
            groups = PCSCReaderGroups().instance
            newgroup = "Pinpad$Readers"
            groups.insert(0, newgroup)
            self.assertEqual(groups, [newgroup] + groupssnapshot)
            groups.remove(newgroup)

        def testcase_removereadergroup_pop(self):
            """Test for groups.pop()"""
            groupssnapshot = list(PCSCReaderGroups().instance)
            groups = PCSCReaderGroups().instance
            newgroup = "Pinpad$Readers"
            groups.insert(0, newgroup)
            self.assertEqual(groups, [newgroup] + groupssnapshot)
            groups.pop(0)
            self.assertEqual(groups, groupssnapshot)

        def testcase_addreadertogroup(self):
            """Test for adding readers to group"""
            groups = PCSCReaderGroups().instance
            newgroup = "Pinpad$Readers"
            groups.insert(0, newgroup)
            for r in PCSCReader.readers("SCard$DefaultReaders"):
                r.addtoreadergroup(newgroup)
            self.assertEqual(
                PCSCReader.readers("SCard$DefaultReaders"), PCSCReader.readers(newgroup)
            )
            groups.pop(0)
            self.assertEqual([], PCSCReader.readers(newgroup))

        def testcase_removereaderfromgroup(self):
            """Test for removing readers from group"""
            groups = PCSCReaderGroups().instance
            newgroup = "Pinpad$Readers"
            groups.insert(0, newgroup)
            for r in PCSCReader.readers("SCard$DefaultReaders"):
                r.addtoreadergroup(newgroup)
            self.assertEqual(
                PCSCReader.readers("SCard$DefaultReaders"), PCSCReader.readers(newgroup)
            )
            for r in PCSCReader.readers("SCard$DefaultReaders"):
                r.removefromreadergroup(newgroup)
            self.assertEqual([], PCSCReader.readers(newgroup))
            groups.pop(0)
            self.assertEqual([], PCSCReader.readers(newgroup))

    def suite():
        suite1 = unittest.defaultTestLoader.loadTestsFromTestCase(testcase_readergroups)
        return unittest.TestSuite(suite1)

else:
    print("These tests are for Windows only")

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python3
"""Unit tests for smartcard.ATR

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import unittest

# gemalto jython
from smartcard.ATR import ATR


class testcase_CAtr(unittest.TestCase):
    """Test APDU class and utilities"""

    def testcase_ATR1(self):
        """Usimera Classic 2."""
        a = ATR(
            [
                0x3B,
                0x9E,
                0x95,
                0x80,
                0x1F,
                0xC3,
                0x80,
                0x31,
                0xA0,
                0x73,
                0xBE,
                0x21,
                0x13,
                0x67,
                0x29,
                0x02,
                0x01,
                0x01,
                0x81,
                0xCD,
                0xB9,
            ]
        )
        historicalbytes = [
            0x80,
            0x31,
            0xA0,
            0x73,
            0xBE,
            0x21,
            0x13,
            0x67,
            0x29,
            0x02,
            0x01,
            0x01,
            0x81,
            0xCD,
        ]
        self.assertEqual(a.getHistoricalBytes(), historicalbytes)
        self.assertEqual(a.getChecksum(), 0xB9)
        self.assertTrue(a.checksumOK)
        self.assertTrue(a.isT0Supported())
        self.assertTrue(not a.isT1Supported())
        self.assertTrue(a.isT15Supported())

    def testcase_ATR2(self):
        """Palmera Protect V2."""
        a = ATR([0x3B, 0x65, 0x00, 0x00, 0x9C, 0x02, 0x02, 0x01, 0x02])
        historicalbytes = [0x9C, 0x02, 0x02, 0x01, 0x02]
        self.assertEqual(a.getHistoricalBytes(), historicalbytes)
        self.assertEqual(a.getChecksum(), None)
        self.assertTrue(a.isT0Supported())
        self.assertTrue(not a.isT1Supported())
        self.assertTrue(not a.isT15Supported())

    def testcase_ATR3(self):
        """Simera 3.13."""
        a = ATR([0x3B, 0x16, 0x18, 0x20, 0x02, 0x01, 0x00, 0x80, 0x0D])
        historicalbytes = [0x20, 0x02, 0x01, 0x00, 0x80, 0x0D]
        self.assertEqual(a.getHistoricalBytes(), historicalbytes)
        self.assertEqual(a.getChecksum(), None)
        self.assertTrue(a.isT0Supported())
        self.assertTrue(not a.isT1Supported())
        self.assertTrue(not a.isT15Supported())

    def testcase_ATR4(self):
        """SIMRock'n Tree"""
        a = ATR(
            [0x3B, 0x77, 0x94, 0x00, 0x00, 0x82, 0x30, 0x00, 0x13, 0x6C, 0x9F, 0x22]
        )
        historicalbytes = [0x82, 0x30, 0x00, 0x13, 0x6C, 0x9F, 0x22]
        self.assertEqual(a.getHistoricalBytes(), historicalbytes)
        self.assertEqual(a.getChecksum(), None)
        self.assertTrue(a.isT0Supported())
        self.assertTrue(not a.isT1Supported())
        self.assertTrue(not a.isT15Supported())

    def testcase_ATR5(self):
        """Demo Vitale online IGEA340"""
        a = ATR([0x3F, 0x65, 0x25, 0x00, 0x52, 0x09, 0x6A, 0x90, 0x00])
        historicalbytes = [0x52, 0x09, 0x6A, 0x90, 0x00]
        self.assertEqual(a.getHistoricalBytes(), historicalbytes)
        self.assertEqual(a.getChecksum(), None)
        self.assertTrue(a.isT0Supported())
        self.assertTrue(not a.isT1Supported())
        self.assertTrue(not a.isT15Supported())

    def testcase_ATR6(self):
        """Simagine 2002"""
        a = ATR([0x3B, 0x16, 0x94, 0x20, 0x02, 0x01, 0x00, 0x00, 0x0D])
        historicalbytes = [0x20, 0x02, 0x01, 0x00, 0x00, 0x0D]
        self.assertEqual(a.getHistoricalBytes(), historicalbytes)
        self.assertEqual(a.getChecksum(), None)

    def testcase_ATR7(self):
        """Protect V3 T=1"""
        a = ATR(
            [
                0x3B,
                0xE5,
                0x00,
                0x00,
                0x81,
                0x21,
                0x45,
                0x9C,
                0x10,
                0x01,
                0x00,
                0x80,
                0x0D,
            ]
        )
        historicalbytes = [0x9C, 0x10, 0x01, 0x00, 0x80]
        self.assertEqual(a.getHistoricalBytes(), historicalbytes)
        self.assertEqual(a.getChecksum(), 0x0D)
        self.assertTrue(not a.isT0Supported())
        self.assertTrue(a.isT1Supported())
        self.assertTrue(not a.isT15Supported())
        self.assertTrue(a.checksumOK)
        self.assertTrue(a.getTB1() == 0x00)
        self.assertTrue(a.getTC1() == 0x00)
        self.assertTrue(a.getTD1() == 0x81)
        self.assertTrue(a.TD[2 - 1] == 0x21)  # TD2
        self.assertTrue(a.TB[3 - 1] == 0x45)  # TB3


if __name__ == "__main__":
    unittest.main(verbosity=1)



#! /usr/bin/env python3
"""
Sample script that illustrates card connection decorators.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from smartcard.CardConnectionDecorator import CardConnectionDecorator
from smartcard.CardConnectionObserver import ConsoleCardConnectionObserver
from smartcard.CardRequest import CardRequest
from smartcard.CardType import AnyCardType
from smartcard.util import toHexString

# define two custom CardConnectionDecorator
# the decorators are very simple, just to illustrate
# shortly how several decorators can be added to the
# card connection


class SecureChannelConnection(CardConnectionDecorator):
    """This decorator is a mockup of secure channel connection.
    It merely pretends to cypher/uncypher upon apdu transmission."""

    def __init__(self, cardconnection):
        CardConnectionDecorator.__init__(self, cardconnection)

    def cypher(self, data):
        """Cypher mock-up; you would include the secure channel logics here."""
        print("cyphering", toHexString(data))
        return data

    def uncypher(self, data):
        """Uncypher mock-up;
        you would include the secure channel logics here."""
        print("uncyphering", toHexString(data))
        return data

    def transmit(self, command, protocol=None):
        """Cypher/uncypher APDUs before transmission"""
        cypheredbytes = self.cypher(command)
        data, sw1, sw2 = CardConnectionDecorator.transmit(self, cypheredbytes, protocol)
        if [] != data:
            data = self.uncypher(data)
        return data, sw1, sw2


class FakeATRConnection(CardConnectionDecorator):
    """This decorator changes the fist byte of the ATR. This is just an example
    to show that decorators can be nested."""

    def __init__(self, cardconnection):
        CardConnectionDecorator.__init__(self, cardconnection)

    def getATR(self):
        """Replace first BYTE of ATR by 3F"""
        atr = CardConnectionDecorator.getATR(self)
        return [0x3F] + atr[1:]


# define the apdus used in this script
GET_RESPONSE = [0xA0, 0xC0, 00, 00]
SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]


# request any card type
cardtype = AnyCardType()
cardrequest = CardRequest(timeout=1.5, cardType=cardtype)
with cardrequest.waitforcard() as cardservice:

    # attach the console tracer
    observer = ConsoleCardConnectionObserver()
    cardservice.connection.addObserver(observer)

    # attach our decorator
    cardservice.connection = FakeATRConnection(
        SecureChannelConnection(cardservice.connection)
    )

    # connect to the card and perform a few transmits
    cardservice.connection.connect()

    print("ATR", toHexString(cardservice.connection.getATR()))

    apdu = SELECT + DF_TELECOM
    response, sw1, sw2 = cardservice.connection.transmit(apdu)

    if sw1 == 0x9F:
        apdu = GET_RESPONSE + [sw2]
        response, sw1, sw2 = cardservice.connection.transmit(apdu)

    cardservice.connection.disconnect()
    cardservice.connection.release()

import sys

if "win32" == sys.platform:
    print("press Enter to continue")
    sys.stdin.read(1)



from __future__ import annotations

import pytest

import smartcard.Synchronization


@pytest.mark.parametrize(
    "defined_methods, names, modified_methods",
    (
        # Nothing should be wrapped (str version)
        pytest.param(set(), "", set(), id="wrap nothing (str)"),
        pytest.param({"a"}, "", set(), id="wrap nothing (a, str)"),
        pytest.param({"a", "b"}, "", set(), id="wrap nothing (a+b, str)"),
        # Nothing should be wrapped (iterable version)
        pytest.param(set(), [], set(), id="wrap nothing (list)"),
        pytest.param({"a"}, [], set(), id="wrap nothing (a, list)"),
        pytest.param({"a", "b"}, [], set(), id="wrap nothing (a+b, list)"),
        # Everything should be wrapped
        pytest.param(set(), None, set(), id="wrap all"),
        pytest.param({"a"}, None, {"a"}, id="wrap all (a)"),
        pytest.param({"a", "b"}, None, {"a", "b"}, id="wrap all (a+b)"),
        # Only "a" should be wrapped (str version)
        pytest.param({"a"}, "a", {"a"}, id="wrap a only (a, str)"),
        pytest.param({"a", "b"}, "a", {"a"}, id="wrap a only (a+b, str)"),
        # Only "a" should be wrapped (list version)
        pytest.param({"a"}, ["a"], {"a"}, id="wrap a only (a, list)"),
        pytest.param({"a", "b"}, ["a"], {"a"}, id="wrap a only (a+b, list)"),
    ),
)
def test_synchronize(
    defined_methods: set[str],
    names: None | str | list[str],
    modified_methods: set[str],
):
    """Verify synchronize() wraps class methods as expected."""

    method_map = {method: lambda self: None for method in defined_methods}
    class_ = type("A", (object,), method_map)

    smartcard.Synchronization.synchronize(class_, names)

    for modified_method in modified_methods:
        assert getattr(class_, modified_method) is not method_map[modified_method]
    for unmodified_method in defined_methods - modified_methods:
        assert getattr(class_, unmodified_method) is method_map[unmodified_method]


def test_synchronization_reentrant_lock():
    """Verify Synchronization mutex locks are re-entrant by default."""

    class A(smartcard.Synchronization.Synchronization):
        def level_1(self):
            self.level_2()

        def level_2(self):
            return self

    smartcard.Synchronization.synchronize(A)

    instance = A()
    # If the synchronization lock is NOT re-entrant by default,
    # the test suite will hang when it reaches this line.
    instance.level_1()


def test_synchronization_wrapping():
    """Verify synchronized functions have correct names and docstrings."""

    class A(smartcard.Synchronization.Synchronization):
        def apple(self):
            """KEEP ME"""

    smartcard.Synchronization.synchronize(A)

    assert A.apple.__name__ == "apple"
    assert "KEEP ME" in A.apple.__doc__


def test_synchronization_kwargs():
    """Verify synchronized functions support arguments and keyword arguments."""

    class A(smartcard.Synchronization.Synchronization):
        def positional_only(self, positional, /):
            return positional

        def keyword_only(self, *, keyword):
            return keyword

    smartcard.Synchronization.synchronize(A)

    A().positional_only(True)
    A().keyword_only(keyword=True)



"""Card service abstract class.

A card service is a class providings specific smart card functionality,
e.g.  a GSM file system or an Open Platform loader.  CardService is an
abstract class from which concrete card services are derived.  A concrete
card service is almost always smart card operating system specific.

The card service performs its specific smart card functionality by accessing
the smartcard with a CardConnection.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from smartcard.scard import *


class CardService:
    """Card service abstract class."""

    def __init__(self, connection, cardname=None):
        """Construct a new card service and bind to a smart card in a reader.

        @param connection:     the CardConnection used to access the smart card
        """
        self.connection = connection
        self.cardname = cardname

    def __del__(self):
        """Destructor. Disconnect card and destroy card service resources."""
        self.__exit__(None, None, None)

    def __enter__(self):
        """Enter the runtime context."""
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        """Exit the runtime context and disconnect + release the PC/SC context."""
        self.connection.disconnect()
        self.connection.release()
        return False

    @staticmethod
    def supports(cardname):
        pass


if __name__ == "__main__":
    """Small sample illustrating the use of CardService."""
    SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
    DF_TELECOM = [0x7F, 0x10]
    from smartcard.System import readers

    cc = readers()[0].createConnection()
    cs = CardService(cc)
    cs.connection.connect()
    data, sw1, sw2 = cs.connection.transmit(SELECT + DF_TELECOM)
    print(f"{sw1:X} {sw2:X}")
    cs.connection.disconnect()
    cs.connection.release()



#! /usr/bin/env python3
"""Unit tests for SCardGetErrorMessage.

This test case can be executed individually, or with all other test cases
thru testsuite_scard.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import sys
import unittest

from smartcard.scard import *


class testcase_geterrormessage(unittest.TestCase):
    """Test scard API for ATR retrieval with SCardLocateCards"""

    def setUp(self):
        hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        self.assertEqual(hresult, 0)

    def tearDown(self):
        hresult = SCardReleaseContext(self.hcontext)
        self.assertEqual(hresult, 0)

    def test_getErrorMessage(self):
        hresult, readers = SCardListReaders(self.hcontext, [])
        self.assertEqual(hresult, 0)

        hresult = SCardReleaseContext(pow(2, 63) >> 60)
        if "win32" == sys.platform:
            self.assertEqual(
                (SCARD_E_INVALID_HANDLE == hresult or ERROR_INVALID_HANDLE == hresult),
                True,
            )
        else:
            self.assertEqual((SCARD_E_INVALID_HANDLE == hresult), True)
        self.assertEqual(
            (
                SCardGetErrorMessage(hresult).rstrip() == "Invalid handle.".rstrip()
                or SCardGetErrorMessage(hresult).rstrip()
                == "The handle is invalid.".rstrip()
            ),
            True,
        )


def suite():
    suite1 = unittest.defaultTestLoader.loadTestsFromTestCase(testcase_geterrormessage)
    return unittest.TestSuite(suite1)


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python3
"""Unit tests for SCardIntroduceCardType/SCardListCards/SCardListInterfaces

This test case can be executed individually, or with all other test cases
thru testsuite_scard.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import platform
import unittest

import smartcard.guid
from smartcard.scard import *

if "winscard" == resourceManager:

    class testcase_listcards(unittest.TestCase):
        """Test scard API for ATR retrieval"""

        # setup for all unit tests: establish context and introduce
        # a dummy card interface
        def setUp(self):
            hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
            self.assertEqual(hresult, 0)
            self.dummycardname = "dummycard"
            self.dummycardATR = [
                0x3B,
                0x75,
                0x94,
                0x00,
                0x00,
                0x62,
                0x02,
                0x02,
                0x01,
                0x01,
            ]
            self.dummycardMask = [
                0xFF,
                0xFF,
                0xFF,
                0xFF,
                0x00,
                0xFF,
                0xFF,
                0xFF,
                0xFF,
                0xFF,
            ]
            self.dummycardguid1 = smartcard.guid.strToGUID(
                "{AD4F1667-EA75-4124-84D4-641B3B197C65}"
            )
            self.dummycardguid2 = smartcard.guid.strToGUID(
                "{382AE95A-7C2C-449c-A179-56C6DE6FF3BC}"
            )
            testcase_listcards.__introduceinterface(self)

        # teardown for all unit tests: release context and forget
        # dummy card interface
        def tearDown(self):
            testcase_listcards.__forgetinterface(self)
            hresult = SCardReleaseContext(self.hcontext)
            self.assertEqual(hresult, 0)

        # introduce a dummy card interface
        # card ATR same as e-gate
        def __introduceinterface(self):
            hresult = SCardForgetCardType(self.hcontext, self.dummycardname)
            dummycardPrimaryGUID = self.dummycardguid1
            dummycardGUIDS = self.dummycardguid1 + self.dummycardguid2
            hresult = SCardIntroduceCardType(
                self.hcontext,
                self.dummycardname,
                dummycardPrimaryGUID,
                dummycardGUIDS,
                self.dummycardATR,
                self.dummycardMask,
            )
            self.assertEqual(hresult, 0)

        # forget dummy card interface
        def __forgetinterface(self):
            hresult = SCardForgetCardType(self.hcontext, self.dummycardname)
            self.assertEqual(hresult, 0)

        # locate a known card
        # Cryptoflex 8k v2 is present in standard Windows 2000
        def test_listcryptoflexbyatr(self):
            slbCryptoFlex8kv2ATR = [
                0x3B,
                0x95,
                0x15,
                0x40,
                0x00,
                0x68,
                0x01,
                0x02,
                0x00,
                0x00,
            ]
            slbCryptoFlex8kv2Name = ["Schlumberger Cryptoflex 8K v2"]
            hresult, card = SCardListCards(self.hcontext, slbCryptoFlex8kv2ATR, [])
            self.assertEqual(hresult, 0)
            self.assertEqual(card, slbCryptoFlex8kv2Name)

        # locate dummy card by interface
        def test_listdummycardbyguid(self):
            guidstolocate = self.dummycardguid2 + self.dummycardguid1
            locatedcardnames = [self.dummycardname]
            hresult, card = SCardListCards(self.hcontext, [], guidstolocate)
            self.assertEqual(hresult, 0)
            self.assertEqual(card, locatedcardnames)

        # list our dummy card interfaces and check
        # that they match the introduced interfaces
        def test_listdummycardinterfaces(self):
            hresult, interfaces = SCardListInterfaces(self.hcontext, self.dummycardname)
            self.assertEqual(hresult, 0)
            self.assertEqual(2, len(interfaces))
            self.assertEqual(self.dummycardguid1, interfaces[0])
            self.assertEqual(self.dummycardguid2, interfaces[1])

        # locate all cards and interfaces in the system
        def test_listallcards(self):

            # dummycard has been introduced in the test setup and
            # will be removed in the test teardown. Other cards are
            # the cards present by default on Windows 2000
            expectedCards = [
                "dummycard",
                "GemSAFE Smart Card (8K)",
                "Schlumberger Cryptoflex 4K",
                "Schlumberger Cryptoflex 8K",
                "Schlumberger Cryptoflex 8K v2",
            ]
            hresult, cards = SCardListCards(self.hcontext, [], [])
            self.assertEqual(hresult, 0)
            foundCards = {}
            for i in range(len(cards)):
                foundCards[cards[i]] = 1
            for i in expectedCards:
                self.assertTrue(i in foundCards)

            # dummycard has a primary provider,
            # other cards have no primary provider
            expectedPrimaryProviderResult = {
                "dummycard": [0, self.dummycardguid1],
                "GemSAFE": [2, None],
                "Schlumberger Cryptoflex 4k": [2, None],
                "Schlumberger Cryptoflex 8k": [2, None],
                "Schlumberger Cryptoflex 8k v2": [2, None],
            }
            for i in range(len(cards)):
                hresult, providername = SCardGetCardTypeProviderName(
                    self.hcontext, cards[i], SCARD_PROVIDER_PRIMARY
                )
                if cards[i] in expectedPrimaryProviderResult:
                    self.assertEqual(
                        hresult, expectedPrimaryProviderResult[cards[i]][0]
                    )
                    if hresult == SCARD_S_SUCCESS:
                        self.assertEqual(
                            providername,
                            smartcard.guid.GUIDToStr(
                                expectedPrimaryProviderResult[cards[i]][1]
                            ),
                        )

            # dummycard has no CSP, other cards have a CSP
            expectedProviderCSPResult = {
                "dummycard": [2, None],
                "GemSAFE": [0, "Gemplus GemSAFE Card CSP v1.0"],
                "Schlumberger Cryptoflex 4k": [
                    0,
                    "Schlumberger Cryptographic Service Provider",
                ],
                "Schlumberger Cryptoflex 8k": [
                    0,
                    "Schlumberger Cryptographic Service Provider",
                ],
                "Schlumberger Cryptoflex 8k v2": [
                    0,
                    "Schlumberger Cryptographic Service Provider",
                ],
            }
            for i in range(len(cards)):
                hresult, providername = SCardGetCardTypeProviderName(
                    self.hcontext, cards[i], SCARD_PROVIDER_CSP
                )
                if cards[i] in expectedProviderCSPResult:
                    self.assertEqual(hresult, expectedProviderCSPResult[cards[i]][0])
                    self.assertEqual(
                        providername, expectedProviderCSPResult[cards[i]][1]
                    )

    def suite():
        suite1 = unittest.defaultTestLoader.loadTestsFromTestCase(testcase_listcards)
        return unittest.TestSuite(suite1)


if __name__ == "__main__":
    unittest.main()



"""wxPython panel display cards/readers as a TreeCtrl.

__author__ = "gemalto https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

# smartcard imports
from threading import RLock

# wxPython GUI modules (https://www.wxpython.org/)
import wx

import smartcard.wx.SimpleSCardApp
from smartcard.CardMonitoring import CardMonitor, CardObserver
from smartcard.Exceptions import CardConnectionException, NoCardException
from smartcard.ReaderMonitoring import ReaderMonitor, ReaderObserver
from smartcard.util import toHexString
from smartcard.wx import ICO_READER, ICO_SMARTCARD


class BaseCardTreeCtrl(wx.TreeCtrl):
    """Base class for the smart card and reader tree controls."""

    def __init__(
        self,
        parent,
        ID=wx.NewId(),
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=0,
        clientpanel=None,
    ):
        """Constructor. Initializes a smartcard or reader tree control."""
        wx.TreeCtrl.__init__(
            self, parent, ID, pos, size, wx.TR_SINGLE | wx.TR_NO_BUTTONS
        )

        self.clientpanel = clientpanel
        self.parent = parent

        isz = (16, 16)
        il = wx.ImageList(isz[0], isz[1])
        self.capindex = il.Add(
            wx.ArtProvider.GetBitmap(wx.ART_HELP_BOOK, wx.ART_OTHER, isz)
        )
        self.fldrindex = il.Add(
            wx.ArtProvider.GetBitmap(wx.ART_FOLDER, wx.ART_OTHER, isz)
        )
        self.fldropenindex = il.Add(
            wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_OTHER, isz)
        )
        if None != ICO_SMARTCARD:
            self.cardimageindex = il.Add(wx.Bitmap(ICO_SMARTCARD, wx.BITMAP_TYPE_ICO))
        if None != ICO_READER:
            self.readerimageindex = il.Add(wx.Bitmap(ICO_READER, wx.BITMAP_TYPE_ICO))
        self.il = il
        self.SetImageList(self.il)

    def Repaint(self):
        self.Refresh()


class CardTreeCtrl(BaseCardTreeCtrl):
    """The CardTreeCtrl monitors inserted cards and notifies the
    application client dialog of any card activation."""

    def __init__(
        self,
        parent,
        ID=wx.NewId(),
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=0,
        clientpanel=None,
    ):
        """Constructor. Create a smartcard tree control."""
        BaseCardTreeCtrl.__init__(
            self, parent, ID, pos, size, wx.TR_SINGLE | wx.TR_NO_BUTTONS, clientpanel
        )

        self.root = self.AddRoot("Smartcards")
        self.SetItemData(self.root, None)
        self.SetItemImage(self.root, self.fldrindex, wx.TreeItemIcon_Normal)
        self.SetItemImage(self.root, self.fldropenindex, wx.TreeItemIcon_Expanded)
        self.Expand(self.root)

    def OnAddCards(self, addedcards):
        """Called when a card is inserted.
        Adds a smart card to the smartcards tree."""
        parentnode = self.root
        for cardtoadd in addedcards:
            childCard = self.AppendItem(parentnode, toHexString(cardtoadd.atr))
            self.SetItemText(childCard, toHexString(cardtoadd.atr))
            self.SetItemData(childCard, cardtoadd)
            self.SetItemImage(childCard, self.cardimageindex, wx.TreeItemIcon_Normal)
            self.SetItemImage(childCard, self.cardimageindex, wx.TreeItemIcon_Expanded)
            self.Expand(childCard)
        self.Expand(self.root)
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnRemoveCards(self, removedcards):
        """Called when a card is removed.
        Removes a card from the tree."""
        parentnode = self.root
        for cardtoremove in removedcards:
            (childCard, cookie) = self.GetFirstChild(parentnode)
            while childCard.IsOk():
                if self.GetItemText(childCard) == toHexString(cardtoremove.atr):
                    self.Delete(childCard)
                (childCard, cookie) = self.GetNextChild(parentnode, cookie)
        self.Expand(self.root)
        self.EnsureVisible(self.root)
        self.Repaint()


class ReaderTreeCtrl(BaseCardTreeCtrl):
    """The ReaderTreeCtrl monitors inserted cards and readers and notifies the
    application client dialog of any card activation."""

    def __init__(
        self,
        parent,
        ID=wx.NewId(),
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=0,
        clientpanel=None,
    ):
        """Constructor. Create a reader tree control."""

        BaseCardTreeCtrl.__init__(
            self, parent, ID, pos, size, wx.TR_SINGLE | wx.TR_NO_BUTTONS, clientpanel
        )

        self.mutex = RLock()

        self.root = self.AddRoot("Smartcard Readers")
        self.SetItemData(self.root, None)
        self.SetItemImage(self.root, self.fldrindex, wx.TreeItemIcon_Normal)
        self.SetItemImage(self.root, self.fldropenindex, wx.TreeItemIcon_Expanded)
        self.Expand(self.root)

    def AddATR(self, readernode, atr):
        """Add an ATR to a reader node."""
        capchild = self.AppendItem(readernode, atr)
        self.SetItemData(capchild, None)
        self.SetItemImage(capchild, self.cardimageindex, wx.TreeItemIcon_Normal)
        self.SetItemImage(capchild, self.cardimageindex, wx.TreeItemIcon_Expanded)
        self.Expand(capchild)
        return capchild

    def GetATR(self, reader):
        """Return the ATR of the card inserted into the reader."""
        atr = "no card inserted"
        try:
            if not type(reader) is str:
                connection = reader.createConnection()
                connection.connect()
                atr = toHexString(connection.getATR())
                connection.disconnect()
        except NoCardException:
            pass
        except CardConnectionException:
            pass
        return atr

    def OnAddCards(self, addedcards):
        """Called when a card is inserted.
        Adds the smart card child to the reader node."""
        self.mutex.acquire()
        try:
            parentnode = self.root
            for cardtoadd in addedcards:
                (childReader, cookie) = self.GetFirstChild(parentnode)
                found = False
                while childReader.IsOk() and not found:
                    if self.GetItemText(childReader) == str(cardtoadd.reader):
                        (childCard, cookie2) = self.GetFirstChild(childReader)
                        self.SetItemText(childCard, toHexString(cardtoadd.atr))
                        self.SetItemData(childCard, cardtoadd)
                        found = True
                    else:
                        (childReader, cookie) = self.GetNextChild(parentnode, cookie)

                # reader was not found, add reader node
                # this happens when card monitoring thread signals
                # added cards before reader monitoring thread signals
                # added readers
                if not found:
                    childReader = self.AppendItem(parentnode, str(cardtoadd.reader))
                    self.SetItemData(childReader, cardtoadd.reader)
                    self.SetItemImage(
                        childReader, self.readerimageindex, wx.TreeItemIcon_Normal
                    )
                    self.SetItemImage(
                        childReader, self.readerimageindex, wx.TreeItemIcon_Expanded
                    )
                    childCard = self.AddATR(childReader, toHexString(cardtoadd.atr))
                    self.SetItemData(childCard, cardtoadd)
                    self.Expand(childReader)

            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnAddReaders(self, addedreaders):
        """Called when a reader is inserted.
        Adds the smart card reader to the smartcard readers tree."""
        self.mutex.acquire()

        try:
            parentnode = self.root
            for readertoadd in addedreaders:
                # is the reader already here?
                found = False
                (childReader, cookie) = self.GetFirstChild(parentnode)
                while childReader.IsOk() and not found:
                    if self.GetItemText(childReader) == str(readertoadd):
                        found = True
                    else:
                        (childReader, cookie) = self.GetNextChild(parentnode, cookie)
                if not found:
                    childReader = self.AppendItem(parentnode, str(readertoadd))
                    self.SetItemData(childReader, readertoadd)
                    self.SetItemImage(
                        childReader, self.readerimageindex, wx.TreeItemIcon_Normal
                    )
                    self.SetItemImage(
                        childReader, self.readerimageindex, wx.TreeItemIcon_Expanded
                    )
                    self.AddATR(childReader, self.GetATR(readertoadd))
                    self.Expand(childReader)
            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnRemoveCards(self, removedcards):
        """Called when a card is removed.
        Removes the card from the tree."""
        self.mutex.acquire()
        try:
            parentnode = self.root
            for cardtoremove in removedcards:
                (childReader, cookie) = self.GetFirstChild(parentnode)
                found = False
                while childReader.IsOk() and not found:
                    if self.GetItemText(childReader) == str(cardtoremove.reader):
                        (childCard, cookie2) = self.GetFirstChild(childReader)
                        self.SetItemText(childCard, "no card inserted")
                        found = True
                    else:
                        (childReader, cookie) = self.GetNextChild(parentnode, cookie)
            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnRemoveReaders(self, removedreaders):
        """Called when a reader is removed.
        Removes the reader from the smartcard readers tree."""
        self.mutex.acquire()
        try:
            parentnode = self.root
            for readertoremove in removedreaders:
                (childReader, cookie) = self.GetFirstChild(parentnode)
                while childReader.IsOk():
                    if self.GetItemText(childReader) == str(readertoremove):
                        self.Delete(childReader)
                    else:
                        (childReader, cookie) = self.GetNextChild(parentnode, cookie)
            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()


class CardAndReaderTreePanel(wx.Panel):
    """Panel containing the smart card and reader tree controls."""

    class _CardObserver(CardObserver):
        """Inner CardObserver. Gets notified of card insertion
        removal by the CardMonitor."""

        def __init__(self, cardtreectrl):
            self.cardtreectrl = cardtreectrl

        def update(self, observable, handlers):
            """CardObserver callback that is notified
            when cards are added or removed."""
            addedcards, removedcards = handlers
            self.cardtreectrl.OnRemoveCards(removedcards)
            self.cardtreectrl.OnAddCards(addedcards)

    class _ReaderObserver(ReaderObserver):
        """Inner ReaderObserver. Gets notified of reader insertion/removal
        by the ReaderMonitor."""

        def __init__(self, readertreectrl):
            self.readertreectrl = readertreectrl

        def update(self, observable, handlers):
            """ReaderObserver callback that is notified when
            readers are added or removed."""
            addedreaders, removedreaders = handlers
            self.readertreectrl.OnRemoveReaders(removedreaders)
            self.readertreectrl.OnAddReaders(addedreaders)

    def __init__(self, parent, appstyle, clientpanel):
        """Constructor. Create a smartcard and reader tree control on the
        left-hand side of the application main frame.
        @param parent: the tree panel parent
        @param appstyle: a combination of the following styles (bitwise or |)
          - TR_SMARTCARD: display a smartcard tree panel
          - TR_READER: display a reader tree panel
          - default is TR_DEFAULT = TR_SMARTCARD
        @param clientpanel: the client panel to notify of smartcard and reader events
        """
        wx.Panel.__init__(self, parent, -1, style=wx.WANTS_CHARS)

        sizer = wx.BoxSizer(wx.VERTICAL)

        # create the smartcard tree
        if appstyle & smartcard.wx.SimpleSCardApp.TR_SMARTCARD:
            self.cardtreectrl = CardTreeCtrl(self, clientpanel=clientpanel)

            # create the smartcard insertion observer
            self.cardtreecardobserver = self._CardObserver(self.cardtreectrl)

            # register as a CardObserver; we will ge
            # notified of added/removed cards
            self.cardmonitor = CardMonitor()
            self.cardmonitor.addObserver(self.cardtreecardobserver)

            sizer.Add(self.cardtreectrl, flag=wx.EXPAND | wx.ALL, proportion=1)

        # create the reader tree
        if appstyle & smartcard.wx.SimpleSCardApp.TR_READER:
            self.readertreectrl = ReaderTreeCtrl(self, clientpanel=clientpanel)

            # create the reader insertion observer
            self.readertreereaderobserver = self._ReaderObserver(self.readertreectrl)

            # register as a ReaderObserver; we will ge
            # notified of added/removed readers
            self.readermonitor = ReaderMonitor()
            self.readermonitor.addObserver(self.readertreereaderobserver)

            # create the smartcard insertion observer
            self.readertreecardobserver = self._CardObserver(self.readertreectrl)

            # register as a CardObserver; we will get
            # notified of added/removed cards
            self.cardmonitor = CardMonitor()
            self.cardmonitor.addObserver(self.readertreecardobserver)

            sizer.Add(self.readertreectrl, flag=wx.EXPAND | wx.ALL, proportion=1)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)

    def OnDestroy(self, event):
        """Called on panel destruction."""
        # deregister observers
        if hasattr(self, "cardmonitor"):
            self.cardmonitor.deleteObserver(self.cardtreecardobserver)
        if hasattr(self, "readermonitor"):
            self.readermonitor.deleteObserver(self.readertreereaderobserver)
            self.cardmonitor.deleteObserver(self.readertreecardobserver)
        event.Skip()



#! /usr/bin/env python3
"""Generates test suite smartcard configuration from
connected readers and cards.

The generated configuration is store in local_config.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import os.path
import time

from smartcard.ATR import ATR
from smartcard.Exceptions import NoCardException
from smartcard.System import readers


def getATR(reader):
    """
    Get card ATR
    """
    cc = reader.createConnection()
    cc.connect()
    atr = cc.getATR()
    cc.disconnect()
    return atr


def checklocalconfig():
    """
    Generate or regenerate local_config.py
    """
    if os.path.isfile("local_config.py"):
        print("regenerating local_config.py...")
    else:
        print("local_config.py not found; generating local_config.py...")

    wrong_setup = False
    # generate local configuration
    with open("local_config.py", "w", encoding="utf-8") as f:

        f.write("from smartcard.util import toHexString\n")
        f.write("expectedReaders = ")
        expectedReaders = readers()
        if len(expectedReaders) < 2:
            print("You should use at least 2 readers.")
            wrong_setup = True
        f.write(str(expectedReaders) + "\n")
        expectedATRs = []
        atr_ok = 0
        for reader in expectedReaders:
            try:
                atr = getATR(reader)
                expectedATRs.append(atr)
                if ATR(atr).isT1Supported():
                    print("You should use T=0 cards")
                    wrong_setup = True
                atr_ok += 1
            except NoCardException:
                expectedATRs.append([])
        if atr_ok < 2:
            print("You should use at least 2 cards.")
            wrong_setup = True

        f.write("expectedATRs = ")
        f.write(repr(expectedATRs) + "\n")

        f.write("expectedATRinReader = {}\n")
        f.write("for i in range(len(expectedReaders)):\n")
        f.write("    expectedATRinReader[expectedReaders[i]] = expectedATRs[i]\n")

        f.write("expectedReaderForATR = {}\n")
        f.write("for i in range(len(expectedReaders)):\n")
        f.write(
            "    expectedReaderForATR[toHexString(expectedATRs[i])] = "
            + "expectedReaders[i]\n"
        )

        f.write("expectedReaderGroups = ['SCard$DefaultReaders']\n")

    if wrong_setup:
        print("The readers or cards setup is invalid. Some tests WILL fail!")
        print("Pause for 5 seconds")
        time.sleep(5)


if __name__ == "__main__":
    import sys

    checklocalconfig()
    sys.exit()



"""Smart card insertion/removal monitoring classes.

CardObserver is a base class for objects that are to be notified
upon smart card insertion/removal.

CardMonitor is a singleton object notifying registered CardObservers
upon reader insertion/removal.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import traceback
from threading import Event, Lock, Thread
from time import sleep

from smartcard.CardRequest import CardRequest
from smartcard.Exceptions import CardRequestTimeoutException, SmartcardException
from smartcard.Observer import Observable, Observer
from smartcard.scard import SCARD_E_NO_SERVICE

_START_ON_DEMAND_ = False


# CardObserver interface
class CardObserver(Observer):
    """
    CardObserver is a base abstract class for objects that are to be notified
    upon smart card insertion / removal.
    """

    def __init__(self):
        pass

    def update(self, observable, handlers):
        """Called upon smart card insertion / removal.

        @param observable:
        @param handlers:
          - addedcards: list of inserted smart cards causing notification
          - removedcards: list of removed smart cards causing notification
        """
        pass


class CardMonitor:
    """Class that monitors smart card insertion / removals.
    and notify observers

    note: a card monitoring thread will be running
    as long as the card monitor has observers, or CardMonitor.stop()
    is called. Do not forget to delete all your observers by
    calling L{deleteObserver}, or your program will run forever...

    Uses the singleton pattern from Thinking in Python
    Bruce Eckel, http://mindview.net/Books/TIPython to make sure
    there is only one L{CardMonitor}.
    """

    class __CardMonitorSingleton(Observable):
        """The real smart card monitor class.

        A single instance of this class is created
        by the public CardMonitor class.
        """

        def __init__(self):
            Observable.__init__(self)
            if _START_ON_DEMAND_:
                self.rmthread = None
            else:
                self.rmthread = CardMonitoringThread(self)

        def addObserver(self, observer):
            """Add an observer.

            We only start the card monitoring thread when
            there are observers.
            """
            Observable.addObserver(self, observer)
            if _START_ON_DEMAND_:
                if self.countObservers() > 0 and self.rmthread is None:
                    self.rmthread = CardMonitoringThread(self)
            else:
                observer.update(self, (self.rmthread.cards, []))

        def deleteObserver(self, observer):
            """Remove an observer.

            We delete the L{CardMonitoringThread} reference when there
            are no more observers.
            """
            Observable.deleteObserver(self, observer)
            if _START_ON_DEMAND_:
                if self.countObservers() == 0:
                    if self.rmthread is not None:
                        self.rmthread.stop()
                        self.rmthread.join()
                        self.rmthread = None

        def __str__(self):
            return "CardMonitor"

    # the singleton
    instance = None
    lock = Lock()

    def __init__(self):
        with CardMonitor.lock:
            if not CardMonitor.instance:
                CardMonitor.instance = CardMonitor.__CardMonitorSingleton()

    def __getattr__(self, name):
        return getattr(self.instance, name)


class CardMonitoringThread:
    """Card insertion thread.
    This thread waits for card insertion.
    """

    class __CardMonitoringThreadSingleton(Thread):
        """The real card monitoring thread class.

        A single instance of this class is created
        by the public L{CardMonitoringThread} class.
        """

        def __init__(self, observable):
            Thread.__init__(self)
            self.observable = observable
            self.stopEvent = Event()
            self.stopEvent.clear()
            self.cards = []
            self.daemon = True

        # the actual monitoring thread
        def run(self):
            """Runs until stopEvent is notified, and notify
            observers of all card insertion/removal.
            """
            self.cardrequest = CardRequest(timeout=60)
            while self.stopEvent.is_set() != 1:
                try:
                    currentcards = self.cardrequest.waitforcardevent()

                    addedcards = []
                    for card in currentcards:
                        if not self.cards.__contains__(card):
                            addedcards.append(card)

                    removedcards = []
                    for card in self.cards:
                        if not currentcards.__contains__(card):
                            removedcards.append(card)

                    if addedcards != [] or removedcards != []:
                        self.cards = currentcards
                        self.observable.setChanged()
                        self.observable.notifyObservers((addedcards, removedcards))

                except CardRequestTimeoutException:
                    pass

                except SmartcardException as exc:
                    # FIXME Tighten the exceptions caught by this block
                    traceback.print_exc()
                    # Most likely raised during interpreter shutdown due
                    # to unclean exit which failed to remove all observers.
                    # To solve this, we set the stop event and pass the
                    # exception to let the thread finish gracefully.
                    if exc.hresult == SCARD_E_NO_SERVICE:
                        self.stopEvent.set()

        # stop the thread by signaling stopEvent
        def stop(self):
            self.stopEvent.set()

    # the singleton
    instance = None

    def __init__(self, observable):
        if not CardMonitoringThread.instance:
            CardMonitoringThread.instance = (
                CardMonitoringThread.__CardMonitoringThreadSingleton(observable)
            )
            CardMonitoringThread.instance.start()

    def join(self, *args, **kwargs):
        if self.instance:
            self.instance.join(*args, **kwargs)
            CardMonitoringThread.instance = None

    def __getattr__(self, name):
        if self.instance:
            return getattr(self.instance, name)


if __name__ == "__main__":
    print("insert or remove cards in the next 10 seconds")

    # a simple card observer that prints added/removed cards
    class printobserver(CardObserver):

        def __init__(self, obsindex):
            self.obsindex = obsindex

        def update(self, observable, handlers):
            addedcards, removedcards = handlers
            print("%d - added:   %s" % (self.obsindex, str(addedcards)))
            print("%d - removed: %s" % (self.obsindex, str(removedcards)))

    class testthread(Thread):

        def __init__(self, obsindex):
            Thread.__init__(self)
            self.readermonitor = CardMonitor()
            self.obsindex = obsindex
            self.observer = None

        def run(self):
            # create and register observer
            self.observer = printobserver(self.obsindex)
            self.readermonitor.addObserver(self.observer)
            sleep(10)
            self.readermonitor.deleteObserver(self.observer)

    t1 = testthread(1)
    t2 = testthread(2)
    t1.start()
    t2.start()



#! /usr/bin/env python3
"""Sample script for APDU error checking.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from smartcard.CardConnectionObserver import ConsoleCardConnectionObserver
from smartcard.CardRequest import CardRequest
from smartcard.CardType import AnyCardType
from smartcard.sw.ErrorCheckingChain import ErrorCheckingChain
from smartcard.sw.ISO7816_4ErrorChecker import ISO7816_4ErrorChecker
from smartcard.sw.ISO7816_8ErrorChecker import ISO7816_8ErrorChecker
from smartcard.sw.ISO7816_9ErrorChecker import ISO7816_9ErrorChecker
from smartcard.sw.SWExceptions import SWException, WarningProcessingException

# define the apdus used in this script
GET_RESPONSE = [0xA0, 0xC0, 00, 00]
SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]


if __name__ == "__main__":

    print("Insert a card within 10 seconds")
    print("Cards without a DF_TELECOM will except")

    # request any card type
    cardtype = AnyCardType()
    cardrequest = CardRequest(timeout=10, cardType=cardtype)
    cardservice = cardrequest.waitforcard()

    # use ISO7816-4 and ISO7816-8 error checking strategy
    # first check iso7816_8 errors, then iso7816_4 errors
    errorchain = []
    errorchain = [ErrorCheckingChain(errorchain, ISO7816_9ErrorChecker())]
    errorchain = [ErrorCheckingChain(errorchain, ISO7816_8ErrorChecker())]
    errorchain = [ErrorCheckingChain(errorchain, ISO7816_4ErrorChecker())]
    cardservice.connection.setErrorCheckingChain(errorchain)

    # filter Warning Processing Exceptions (sw1 = 0x62 or 0x63)
    cardservice.connection.addSWExceptionToFilter(WarningProcessingException)

    # attach the console tracer
    observer = ConsoleCardConnectionObserver()
    cardservice.connection.addObserver(observer)

    # connect to the card and perform a few transmits
    cardservice.connection.connect()

    try:
        apdu = SELECT + DF_TELECOM
        response, sw1, sw2 = cardservice.connection.transmit(apdu)

        if sw1 == 0x9F:
            apdu = GET_RESPONSE + [sw2]
            response, sw1, sw2 = cardservice.connection.transmit(apdu)

    except SWException as e:
        print(str(e))

    cardservice.connection.disconnect()
    cardservice.connection.release()

    import sys

    if "win32" == sys.platform:
        print("press Enter to continue")
        sys.stdin.read(1)



import re
import textwrap

import pytest

from smartcard.ATR import ATR
from smartcard.Exceptions import SmartcardException
from smartcard.util import toBytes


def test_atr1(capsys):
    atr = [0x3F, 0x65, 0x25, 0x00, 0x2C, 0x09, 0x69, 0x90, 0x00]
    data_out = textwrap.dedent(
        """\
        TB1: 25
        TC1: 0
        supported protocols T=0
        T=0 supported: True
        T=1 supported: False
        \tclock rate conversion factor: 372
        \tbit rate adjustment factor: 1
        \tmaximum programming current: 50
        \tprogramming voltage: 30
        \tguard time: 0
        nb of interface bytes: 2
        nb of historical bytes: 5
        """
    )
    a = ATR(atr)
    with pytest.warns(DeprecationWarning, match=re.escape("print(ATR.render())")):
        a.dump()
    stdout, _ = capsys.readouterr()
    assert stdout == data_out


def test_atr2(capsys):
    atr = [0x3F, 0x65, 0x25, 0x08, 0x93, 0x04, 0x6C, 0x90, 0x00]
    data_out = textwrap.dedent(
        """\
        TB1: 25
        TC1: 8
        supported protocols T=0
        T=0 supported: True
        T=1 supported: False
        \tclock rate conversion factor: 372
        \tbit rate adjustment factor: 1
        \tmaximum programming current: 50
        \tprogramming voltage: 30
        \tguard time: 8
        nb of interface bytes: 2
        nb of historical bytes: 5
        """
    )
    a = ATR(atr)
    with pytest.warns(DeprecationWarning, match=re.escape("print(ATR.render())")):
        a.dump()
    stdout, _ = capsys.readouterr()
    assert stdout == data_out


def test_atr3(capsys):
    atr = [0x3B, 0x16, 0x94, 0x7C, 0x03, 0x01, 0x00, 0x00, 0x0D]
    data_out = textwrap.dedent(
        """\
        TA1: 94
        supported protocols T=0
        T=0 supported: True
        T=1 supported: False
        \tclock rate conversion factor: 512
        \tbit rate adjustment factor: 8
        \tmaximum programming current: 50
        \tprogramming voltage: 5
        \tguard time: None
        nb of interface bytes: 1
        nb of historical bytes: 6
        """
    )
    a = ATR(atr)
    with pytest.warns(DeprecationWarning, match=re.escape("print(ATR.render())")):
        a.dump()
    stdout, _ = capsys.readouterr()
    assert stdout == data_out


def test_atr4(capsys):
    atr = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
    data_out = textwrap.dedent(
        """\
        TB1: 0
        TC1: 0
        supported protocols T=0
        T=0 supported: True
        T=1 supported: False
        \tclock rate conversion factor: 372
        \tbit rate adjustment factor: 1
        \tmaximum programming current: 25
        \tprogramming voltage: 5
        \tguard time: 0
        nb of interface bytes: 2
        nb of historical bytes: 5
        """
    )
    a = ATR(atr)
    with pytest.warns(DeprecationWarning, match=re.escape("print(ATR.render())")):
        a.dump()
    stdout, _ = capsys.readouterr()
    assert stdout == data_out


def test_atr5(capsys):
    atr = [0x3B, 0xE3, 0x00, 0xFF, 0x81, 0x31, 0x52, 0x45, 0xA1, 0xA2, 0xA3, 0x1B]
    data_out = textwrap.dedent(
        """\
        TB1: 0
        TC1: ff
        TD1: 81
        TD2: 31
        TA3: 52
        TB3: 45
        supported protocols T=1
        T=0 supported: False
        T=1 supported: True
        checksum: 27
        \tclock rate conversion factor: 372
        \tbit rate adjustment factor: 1
        \tmaximum programming current: 25
        \tprogramming voltage: 5
        \tguard time: 255
        nb of interface bytes: 6
        nb of historical bytes: 3
        """
    )
    a = ATR(atr)
    with pytest.warns(DeprecationWarning, match=re.escape("print(ATR.render())")):
        a.dump()
    stdout, _ = capsys.readouterr()
    assert stdout == data_out


def test_atr6(capsys):
    atr = [0x3B, 0xE5, 0x00, 0x00, 0x81, 0x21, 0x45, 0x9C, 0x10, 0x01, 0x00, 0x80, 0x0D]
    data_out = textwrap.dedent(
        """\
        TB1: 0
        TC1: 0
        TD1: 81
        TD2: 21
        TB3: 45
        supported protocols T=1
        T=0 supported: False
        T=1 supported: True
        checksum: 13
        \tclock rate conversion factor: 372
        \tbit rate adjustment factor: 1
        \tmaximum programming current: 25
        \tprogramming voltage: 5
        \tguard time: 0
        nb of interface bytes: 5
        nb of historical bytes: 5
        """
    )
    a = ATR(atr)
    with pytest.warns(DeprecationWarning, match=re.escape("print(ATR.render())")):
        a.dump()
    stdout, _ = capsys.readouterr()
    assert stdout == data_out


@pytest.mark.parametrize(
    "ts",
    (
        pytest.param("0x42", id="numeric"),
        pytest.param("0xaa", id="lowercase"),
        pytest.param("0x00", id="zero padding"),
    ),
)
def test_invalid_ts(ts: str):
    atr = [int(ts[2:], 16), 0x00]
    with pytest.raises(SmartcardException, match=f"invalid TS {ts}"):
        ATR(atr)


def test_atr_get():
    atr = "3B F2 95 12 34 01 36 06"
    a = ATR(toBytes(atr))
    assert a.getTA1() == 0x95
    assert a.getTB1() == 0x12
    assert a.getTC1() == 0x34
    assert a.getTD1() == 0x01
    assert a.getHistoricalBytes(), [0x36 == 0x06]
    assert a.isT15Supported() is False
    assert str(a) == atr


@pytest.mark.parametrize(
    "field, expected_length",
    (
        ("clockrateconversion", 16),
        ("bitratefactor", 16),
    ),
)
def test_map_lengths(field, expected_length):
    """Verify ATR class fields have expected lengths.

    This doesn't validate values, but simply ensures the lengths match expectations.
    """

    assert len(getattr(ATR, field)) == expected_length


@pytest.mark.parametrize(
    "atr,",
    (
        pytest.param([], id="ATR is too short (0 bytes)"),
        pytest.param([0x3B], id="ATR is too short (1 byte, valid TS)"),
    ),
)
def test_invalid_atr_lengths(atr: list[int]):
    """Verify that short ATRs raise exceptions."""

    with pytest.raises(SmartcardException, match="at least 2 bytes"):
        ATR(atr)


@pytest.mark.parametrize("ts", (0x3B, 0x3F))
def test_2_bytes(ts):
    """Verify that a completely empty ATR parses well."""

    atr = ATR([ts, 0b0000_0000])
    #                |||| `-- no historical bytes
    #                |||`-- no TA
    #                ||`-- no TB
    #                |`-- no TC
    #                `-- no TD
    assert atr.getTA1() is None
    assert atr.getTB1() is None
    assert atr.II is None
    assert atr.PI1 is None
    assert atr.getTC1() is None
    assert atr.getTD1() is None
    assert atr.getChecksum() is None
    assert atr.getGuardTime() is None
    assert atr.getHistoricalBytesCount() == 0
    assert atr.getHistoricalBytes() == []
    assert atr.getInterfaceBytesCount() == 0

    # Default values
    assert atr.getBitRateFactor() == 1
    assert atr.getClockRateConversion() == 372
    assert atr.getProgrammingCurrent() == 50
    assert atr.getProgrammingVoltage() == 5

    # Protocols
    assert len(atr.getSupportedProtocols()) == 1
    assert "T=0" in atr.getSupportedProtocols()
    assert atr.isT0Supported() is True
    assert atr.isT1Supported() is False
    assert atr.isT15Supported() is False

    # Rendering
    expected_rendering = textwrap.dedent(
        """\
        supported protocols T=0
        T=0 supported: True
        T=1 supported: False
        \tclock rate conversion factor: 372
        \tbit rate adjustment factor: 1
        \tmaximum programming current: 50
        \tprogramming voltage: 5
        \tguard time: None
        nb of interface bytes: 0
        nb of historical bytes: 0
        """.rstrip()
    )
    assert atr.render() == expected_rendering

    # Warnings
    with pytest.warns(DeprecationWarning, match="ATR.TA"):
        assert atr.hasTA == [False]
    with pytest.warns(DeprecationWarning, match="ATR.TB"):
        assert atr.hasTB == [False]
    with pytest.warns(DeprecationWarning, match="ATR.TC"):
        assert atr.hasTC == [False]
    with pytest.warns(DeprecationWarning, match="ATR.TD"):
        assert atr.hasTD == [False]


def test_only_ta1():
    """Verify that TA1 can be conveyed standalone."""

    atr = ATR([0x3B, 0b0001_0000, 0xA7])
    #                     `-- only enable TA
    assert atr.TA == [0xA7]
    assert "TA1: a7\n" in atr.render()
    with pytest.warns(DeprecationWarning, match="ATR.TA"):
        assert atr.hasTA == [True]
    # TA1 affects these values
    assert atr.getClockRateConversion() == 768
    assert atr.getBitRateFactor() == 64
    # Sanity check
    assert atr.TB == atr.TC == atr.TD == [None]
    assert atr.N is None
    assert atr.getInterfaceBytesCount() == 1
    assert atr.getHistoricalBytesCount() == 0
    assert atr.hasChecksum is False
    assert atr.checksumOK is None
    assert atr.getChecksum() is None


def test_only_tb1():
    """Verify that TB1 can be conveyed standalone.

    TB1 and TB2 are deprecated in ISO 7816-3 2006, so no values are checked here.
    """

    atr = ATR([0x3B, 0b0010_0000, 0b0_10_11111])
    #                    `-- only enable TB
    assert atr.TB == [0b0_10_11111]
    assert "TB1: 5f\n" in atr.render()
    with pytest.warns(DeprecationWarning, match="ATR.TB"):
        assert atr.hasTB == [True]
    # TB1 affects these values
    assert atr.II == 0b10
    assert atr.PI1 == 0b11111
    assert atr.getProgrammingVoltage() != 5
    assert atr.getProgrammingCurrent() != 50
    # Sanity check
    assert atr.TA == atr.TC == atr.TD == [None]
    assert atr.N is None
    assert atr.getInterfaceBytesCount() == 1
    assert atr.getHistoricalBytesCount() == 0
    assert atr.hasChecksum is False
    assert atr.checksumOK is None
    assert atr.getChecksum() is None


def test_only_tc1():
    """Verify that TC1 can be conveyed standalone."""

    atr = ATR([0x3B, 0b0100_0000, 0xC1])
    #                   `-- only enable TC
    assert atr.TC == [0xC1]
    assert "TC1: c1\n" in atr.render()
    with pytest.warns(DeprecationWarning, match="ATR.TC"):
        assert atr.hasTC == [True]
    # TC1 affects these values
    assert atr.N == 0xC1
    # Sanity check
    assert atr.TA == atr.TB == atr.TD == [None]
    assert atr.getInterfaceBytesCount() == 1
    assert atr.getHistoricalBytesCount() == 0
    assert atr.hasChecksum is False
    assert atr.checksumOK is None
    assert atr.getChecksum() is None


def test_only_td1():
    """Verify that TD1 can be conveyed standalone."""

    atr = ATR([0x3B, 0b1000_0000, 0x00])
    #                  `-- only enable TD
    assert atr.TD == [0x00, None]
    assert atr.isT0Supported() is True
    assert atr.isT1Supported() is False
    assert atr.isT15Supported() is False
    assert "TD1: 0\n" in atr.render()
    with pytest.warns(DeprecationWarning, match="ATR.TD"):
        assert atr.hasTD == [True, False]
    # Sanity check
    assert atr.TA == atr.TB == atr.TC == [None, None]
    assert atr.N is None
    assert atr.getHistoricalBytesCount() == 0
    assert atr.hasChecksum is False
    assert atr.checksumOK is None
    assert atr.getChecksum() is None


def test_historical_bytes():
    """Verify that historical bytes can be conveyed standalone."""

    atr = ATR([0x3B, 0x0F, *list(range(15))])
    #                   `-- indicate 15 historical bytes
    assert atr.K == 15
    assert atr.getHistoricalBytesCount() == 15
    assert atr.getHistoricalBytes() == list(range(15))
    # Sanity check
    assert atr.TA == atr.TB == atr.TC == atr.TD == [None]
    assert atr.N is None
    assert atr.hasChecksum is False
    assert atr.checksumOK is None
    assert atr.getChecksum() is None


@pytest.mark.parametrize("ts", (0x3B, 0x3F))
@pytest.mark.parametrize("atr_bytes", ([0x00, 0x00], [0x1, 0xFE, 0xFF]))
def test_valid_checksums(ts, atr_bytes):
    """Verify behavior of valid checksums."""

    atr = ATR([ts] + atr_bytes)
    assert atr.hasChecksum is True
    assert atr.checksumOK is True
    assert atr.getChecksum() == atr_bytes[-1]
    assert f"checksum: {atr_bytes[-1]}\n" in atr.render()


@pytest.mark.parametrize("ts", (0x3B, 0x3F))
@pytest.mark.parametrize("atr_bytes", ([0x00, 0x01], [0x01, 0xFE, 0x0]))
def test_invalid_checksums(ts, atr_bytes):
    """Verify behavior of invalid checksums."""

    atr = ATR([ts] + atr_bytes)
    assert atr.hasChecksum is True
    assert atr.checksumOK is False
    assert atr.getChecksum() == atr_bytes[-1]
    assert f"checksum: {atr_bytes[-1]:x}\n" in atr.render()



#! /usr/bin/env python3
"""Unit tests for SCardxxx readers and readergroups methods.

This test case can be executed individually, or with all other test cases
thru testsuite_scard.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
import unittest

from smartcard.scard import *

sys.path += [".."]

try:
    from local_config import expectedReaders
except ImportError:
    print("execute test suite first to generate the local_config.py file")
    sys.exit()


expectedGroups = ["SCard$DefaultReaders"]


class testcase_readergroups(unittest.TestCase):
    """Test scard reader groups API"""

    def setUp(self):
        hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        self.assertEqual(hresult, 0)

    def tearDown(self):
        hresult = SCardReleaseContext(self.hcontext)
        self.assertEqual(hresult, 0)

    def test_listReaders(self):

        # list current readers and compare with expected list
        hresult, readers = SCardListReaders(self.hcontext, [])
        self.assertEqual(hresult, 0)
        for i in range(len(expectedReaders)):
            self.assertEqual(readers[i], expectedReaders[i])

        # list current reader groups and compare with expected list
        hresult, readerGroups = SCardListReaderGroups(self.hcontext)
        self.assertEqual(hresult, 0)
        for i in range(len(expectedGroups)):
            self.assertEqual(readerGroups[i], expectedGroups[i])

        if "winscard" == resourceManager:
            # add a new group
            newgroup = "SCard$MyOwnGroup"
            expectedGroups.append(newgroup)

            hresult = SCardIntroduceReaderGroup(self.hcontext, newgroup)
            self.assertEqual(hresult, 0)

            dummyreader = readers[0] + " alias"
            hresult = SCardIntroduceReader(self.hcontext, dummyreader, readers[0])
            self.assertEqual(hresult, 0)

            hresult = SCardAddReaderToGroup(self.hcontext, dummyreader, newgroup)
            self.assertEqual(hresult, 0)

            hresult, readerGroups = SCardListReaderGroups(self.hcontext)
            self.assertEqual(hresult, 0)
            for i in range(len(expectedGroups)):
                self.assertEqual(readerGroups[i], expectedGroups[i])

            # list readers in new group
            hresult, newreaders = SCardListReaders(self.hcontext, [newgroup])
            self.assertEqual(hresult, 0)
            self.assertEqual(newreaders[0], dummyreader)

            # remove reader from new group
            hresult = SCardRemoveReaderFromGroup(self.hcontext, dummyreader, newgroup)
            self.assertEqual(hresult, 0)

            hresult, readerGroups = SCardListReaderGroups(self.hcontext)
            self.assertEqual(hresult, 0)

            expectedGroups.remove(newgroup)
            for i in range(len(expectedGroups)):
                self.assertEqual(readerGroups[i], expectedGroups[i])

            hresult = SCardForgetReaderGroup(self.hcontext, newgroup)
            self.assertEqual(hresult, 0)

            hresult, readerGroups = SCardListReaderGroups(self.hcontext)
            self.assertEqual(hresult, 0)
            for i in range(len(expectedGroups)):
                self.assertEqual(readerGroups[i], expectedGroups[i])

            hresult = SCardForgetReader(self.hcontext, dummyreader)
            self.assertEqual(hresult, 0)


def suite():
    suite1 = unittest.defaultTestLoader.loadTestsFromTestCase(testcase_readergroups)
    return unittest.TestSuite(suite1)


if __name__ == "__main__":
    # When this module is executed from the command-line, run all its tests
    unittest.main()



#! /usr/bin/env python3
"""Unit tests for smartcard.CardRequest

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
import unittest

sys.path += [".."]

try:
    from local_config import (
        expectedATRinReader,
        expectedATRs,
        expectedReaderForATR,
        expectedReaders,
    )
except ImportError:
    print("execute test suite first to generate the local_config.py file")
    sys.exit()


import smartcard.System
from smartcard.CardRequest import CardRequest
from smartcard.CardType import AnyCardType, ATRCardType
from smartcard.Exceptions import CardRequestTimeoutException
from smartcard.PassThruCardService import PassThruCardService
from smartcard.util import toHexString


class testcase_CardRequest(unittest.TestCase):
    """Test case for CardType."""

    def testcase_CardRequestATRCardType(self):
        """Test smartcard.AnyCardType."""

        for atr in expectedATRs:
            if atr:
                ct = ATRCardType(atr)
                cr = CardRequest(timeout=10, cardType=ct)
                cs = cr.waitforcard()
                cs.connection.connect()
                self.assertEqual(atr, cs.connection.getATR())
                self.assertEqual(
                    cs.connection.getReader(), expectedReaderForATR[toHexString(atr)]
                )
                cs.connection.disconnect()
                cs.connection.release()

    def testcase_CardRequestAnyCardTypeInSelectedReader(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            atr = expectedATRinReader[reader]
            if atr:
                ct = AnyCardType()
                cr = CardRequest(timeout=10.6, readers=[reader], cardType=ct)
                cs = cr.waitforcard()
                cs.connection.connect()
                self.assertEqual(atr, cs.connection.getATR())
                self.assertEqual(
                    cs.connection.getReader(), expectedReaderForATR[toHexString(atr)]
                )
                cs.connection.disconnect()
                cs.connection.release()

    def testcase_CardRequestATRCardTypeTimeout(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            atr = expectedATRinReader[reader][:-1]
            ct = ATRCardType(atr)
            cr = CardRequest(timeout=1, readers=[reader], cardType=ct)
            self.assertRaises(CardRequestTimeoutException, cr.waitforcard)

    def testcase_CardRequestATRCardTypeTimeoutAnyReader(self):
        """Test smartcard.AnyCardType."""

        readers = smartcard.System.readers()
        atr = expectedATRs[0][:-1]
        ct = ATRCardType(atr)
        cr = CardRequest(timeout=1.5, readers=readers, cardType=ct)
        self.assertRaises(CardRequestTimeoutException, cr.waitforcard)

    def testcase_CardRequestAnyCardTypeAnyReaderPassThru(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            atr = expectedATRinReader[reader]
            if atr:
                ct = AnyCardType()
                cardservice = PassThruCardService
                cr = CardRequest(
                    timeout=10.6,
                    readers=[reader],
                    cardType=ct,
                    cardServiceClass=cardservice,
                )
                cs = cr.waitforcard()
                cs.connection.connect()
                self.assertEqual(cs.__class__, PassThruCardService)
                self.assertEqual(atr, cs.connection.getATR())
                self.assertEqual(
                    cs.connection.getReader(), expectedReaderForATR[toHexString(atr)]
                )
                cs.connection.disconnect()
                cs.connection.release()

    def testcase_CardRequestAnyCardTypeInSelectedReaderNewCard(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            ct = AnyCardType()
            cr = CardRequest(newcardonly=True, timeout=1, readers=[reader], cardType=ct)
            self.assertRaises(CardRequestTimeoutException, cr.waitforcard)


if __name__ == "__main__":
    unittest.main(verbosity=1)



#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: Locate cards in the system

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import sys

from smartcard.scard import *

if "winscard" == resourceManager:

    znewcardName = "dummy-card"
    znewcardATR = [
        0x3B,
        0x77,
        0x94,
        0x00,
        0x00,
        0x82,
        0x30,
        0x00,
        0x13,
        0x6C,
        0x9F,
        0x22,
    ]
    znewcardMask = [
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
    ]

    try:
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise scard.error(
                "Failed to establish context: " + SCardGetErrorMessage(hresult)
            )
        print("Context established!")

        try:
            hresult, readers = SCardListReaders(hcontext, [])
            if hresult != SCARD_S_SUCCESS:
                raise scard.error(
                    "Failed to list readers: " + SCardGetErrorMessage(hresult)
                )
            print("PCSC Readers:", readers)

            # introduce a card (forget first in case it is already present)
            hresult = SCardForgetCardType(hcontext, znewcardName)
            print("Introducing card " + znewcardName)
            hresult = SCardIntroduceCardType(
                hcontext, znewcardName, [], [], znewcardATR, znewcardMask
            )
            if hresult != SCARD_S_SUCCESS:
                if hresult == ERROR_ALREADY_EXISTS:
                    print("Card already exists")
                else:
                    raise error(
                        "Failed to introduce card type: "
                        + SCardGetErrorMessage(hresult)
                    )

            hresult, cards = SCardListCards(hcontext, [], [])
            if hresult != SCARD_S_SUCCESS:
                raise error("Failure to list cards")
            print("Cards:", cards)

            readerstates = []
            for i in range(len(readers)):
                readerstates += [(readers[i], SCARD_STATE_UNAWARE)]
            print(readerstates)

            hresult, newstates = SCardLocateCards(hcontext, cards, readerstates)
            for i in newstates:
                reader, eventstate, atr = i
                print(reader, end=" ")
                for b in atr:
                    print("0x%.2X" % b, end=" ")
                print("")
                if eventstate & SCARD_STATE_ATRMATCH:
                    print("Card found")
                if eventstate & SCARD_STATE_UNAWARE:
                    print("State unware")
                if eventstate & SCARD_STATE_IGNORE:
                    print("Ignore reader")
                if eventstate & SCARD_STATE_UNAVAILABLE:
                    print("Reader unavailable")
                if eventstate & SCARD_STATE_EMPTY:
                    print("Reader empty")
                if eventstate & SCARD_STATE_PRESENT:
                    print("Card present in reader")
                if eventstate & SCARD_STATE_EXCLUSIVE:
                    print("Card allocated for exclusive use")
                if eventstate & SCARD_STATE_INUSE:
                    print("Card in use but can be shared")
                if eventstate & SCARD_STATE_MUTE:
                    print("Card is mute")
                if eventstate & SCARD_STATE_CHANGED:
                    print("State changed")
                if eventstate & SCARD_STATE_UNKNOWN:
                    print("State unknowned")

        finally:
            hresult = SCardForgetCardType(hcontext, znewcardName)
            hresult = SCardReleaseContext(hcontext)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    "Failed to release context: " + SCardGetErrorMessage(hresult)
                )
            print("Released context.")

    except error as e:
        print(e)

elif "pcsclite" == resourceManager:
    print("SCardLocateCards not supported by pcsc lite")


if "win32" == sys.platform:
    print("press Enter to continue")
    sys.stdin.read(1)



#! /usr/bin/env python3
"""
Sample script for the smartcard.ATR utility class.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2009 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from smartcard.ATR import ATR
from smartcard.util import toHexString

atr = ATR(
    [
        0x3B,
        0x9E,
        0x95,
        0x80,
        0x1F,
        0xC3,
        0x80,
        0x31,
        0xA0,
        0x73,
        0xBE,
        0x21,
        0x13,
        0x67,
        0x29,
        0x02,
        0x01,
        0x01,
        0x81,
        0xCD,
        0xB9,
    ]
)

print(atr)
print("historical bytes: ", toHexString(atr.getHistoricalBytes()))
print("checksum: ", "0x%X" % atr.getChecksum())
print("checksum OK: ", atr.checksumOK)
print("T0  supported: ", atr.isT0Supported())
print("T1  supported: ", atr.isT1Supported())
print("T15 supported: ", atr.isT15Supported())



#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: perform a simple transaction

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from smartcard.scard import *

try:
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    if hresult != SCARD_S_SUCCESS:
        raise error("Failed to establish context: " + SCardGetErrorMessage(hresult))
    print("Context established!")

    try:
        hresult, readers = SCardListReaders(hcontext, [])
        if hresult != SCARD_S_SUCCESS:
            raise error("Failed to list readers: " + SCardGetErrorMessage(hresult))
        print("PCSC Readers:", readers)

        if len(readers) < 1:
            raise error("No smart card readers")

        for zreader in readers:
            print("Trying to perform transaction on card in", zreader)

            try:
                hresult, hcard, dwActiveProtocol = SCardConnect(
                    hcontext,
                    zreader,
                    SCARD_SHARE_SHARED,
                    SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1,
                )
                if hresult != SCARD_S_SUCCESS:
                    raise error("unable to connect: " + SCardGetErrorMessage(hresult))
                print("Connected with active protocol", dwActiveProtocol)

                try:
                    hresult = SCardBeginTransaction(hcard)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            "failed to begin transaction: "
                            + SCardGetErrorMessage(hresult)
                        )
                    print("Beginning transaction")

                    hresult, reader, state, protocol, atr = SCardStatus(hcard)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            "failed to get status: " + SCardGetErrorMessage(hresult)
                        )
                    print("ATR:", end=" ")
                    for i in range(len(atr)):
                        print("0x%.2X" % atr[i], end=" ")
                    print("")

                finally:
                    hresult = SCardEndTransaction(hcard, SCARD_LEAVE_CARD)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            "failed to end transaction: "
                            + SCardGetErrorMessage(hresult)
                        )
                    print("Transaction ended")

                    hresult = SCardDisconnect(hcard, SCARD_UNPOWER_CARD)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            "failed to disconnect: " + SCardGetErrorMessage(hresult)
                        )
                    print("Disconnected")
            except error as message:
                print(error, message)

    finally:
        hresult = SCardReleaseContext(hcontext)
        if hresult != SCARD_S_SUCCESS:
            raise error("failed to release context: " + SCardGetErrorMessage(hresult))
        print("Released context.")

except error as e:
    print(e)

import sys

if "win32" == sys.platform:
    print("press Enter to continue")
    sys.stdin.read(1)



import pytest

import smartcard.reader.ReaderGroups
import smartcard.ulist


def test_reader_groups_acts_like_a_singleton():
    """Verify that the target class *acts* like a singleton."""

    instance_1 = smartcard.reader.ReaderGroups.readergroups()
    instance_2 = smartcard.reader.ReaderGroups.readergroups()
    assert instance_1.instance is instance_2.instance


def test_reader_groups_method_calls():
    reader_group = smartcard.reader.ReaderGroups.readergroups()
    assert len(reader_group.instance) == 0, "No reader groups should be pre-defined"

    list.append(reader_group.instance, "a")
    assert len(reader_group.instance) == 1, "Unable to add a reader group"

    reader_group.addreadergroup("a")
    assert len(reader_group.instance) == 1, "Reader groups must be unique"

    with pytest.raises(smartcard.reader.ReaderGroups.BadReaderGroupException):
        reader_group.addreadergroup(1)
    with pytest.raises(smartcard.reader.ReaderGroups.BadReaderGroupException):
        reader_group.removereadergroup(1)

    # .__iter__()
    assert len(list(reader_group.instance)) == 1

    # No-op calls
    reader_group.addreadertogroup("no-op", "bogus")
    reader_group.removereaderfromgroup("no-op", "bogus")

    assert list.pop(reader_group.instance) == "a"
    assert len(list(reader_group.instance)) == 0


# ------------------------------------------------------------------------------------
# The tests below this line demonstrate serious behavioral problems with readergroups.


@pytest.mark.xfail(reason="readergroups is not actually a singleton", strict=True)
def test_reader_groups_is_a_singleton():
    """Verify that readergroups is a singleton."""

    instance_1 = smartcard.reader.ReaderGroups.readergroups()
    instance_2 = smartcard.reader.ReaderGroups.readergroups()
    assert instance_1 is instance_2


@pytest.mark.xfail(reason="initlist parameters may be silently ignored", strict=True)
def test_demonstrate_initlist_values_may_be_silently_ignored():
    """Demonstrate that `initlist` parameters may be silently ignored."""

    smartcard.reader.ReaderGroups.readergroups(["a"])
    reader_group = smartcard.reader.ReaderGroups.readergroups(["b"])
    assert "b" in reader_group.instance


def test_demonstrate_adding_is_impossible():
    """Demonstrate that `.addreadergroup()` cannot be called."""

    reader_group = smartcard.reader.ReaderGroups.readergroups()
    with pytest.raises(RecursionError, match="maximum recursion depth exceeded"):
        reader_group.addreadergroup("a")


def test_demonstrate_removing_is_impossible():
    """Demonstrate that `.removereadergroup()` cannot be called.

    `.removereadergroup()` recurses twice.
    It successfully removes the value, then fails to find it the second time.
    """

    reader_group = smartcard.reader.ReaderGroups.readergroups()
    list.append(reader_group.instance, "a")
    assert reader_group.instance == ["a"]
    with pytest.raises(ValueError, match="x not in list"):
        reader_group.removereadergroup("a")
    assert reader_group.instance == []


def test_demonstrate_getting_is_impossible():
    """Demonstrate that `.getreadergroups()` returns hard-coded values."""

    reader_group = smartcard.reader.ReaderGroups.readergroups()
    list.append(reader_group.instance, "a")
    assert reader_group.instance == ["a"]
    assert reader_group.getreadergroups() == []
    assert list.pop(reader_group.instance) == "a"
    assert len(list(reader_group.instance)) == 0



"""CardConnectionObserver interface.

CardConnectionObserver is a base class for objects that are to be notified
upon CardConnection events.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from smartcard.Observer import Observer
from smartcard.util import toHexString


# ReaderObserver interface
class CardConnectionObserver(Observer):
    """
    CardConnectionObserver is a base class for objects that are to be notified
    upon L{CardConnection} events.
    """

    def update(self, cardconnection, cardconnectionevent):
        """Called upon CardConnection event.

        @param cardconnection:         the observed card connection object
        @param cardconnectionevent:    the CardConnectionEvent sent by the connection
        """
        pass


class ConsoleCardConnectionObserver(CardConnectionObserver):

    def update(self, cardconnection, ccevent):

        if "connect" == ccevent.type:
            print("connecting to " + cardconnection.getReader())

        elif "reconnect" == ccevent.type:
            print("reconnecting to " + cardconnection.getReader())

        elif "disconnect" == ccevent.type:
            print("disconnecting from " + cardconnection.getReader())

        elif "release" == ccevent.type:
            print("release from " + cardconnection.getReader())

        elif "command" == ccevent.type:
            print("> " + toHexString(ccevent.args[0]))

        elif "response" == ccevent.type:
            if [] == ccevent.args[0]:
                print("<  [] %02X %02X" % tuple(ccevent.args[-2:]))
            else:
                print(
                    "< "
                    + toHexString(ccevent.args[0])
                    + " "
                    + "%02X %02X" % tuple(ccevent.args[-2:])
                )
        else:
            print("unknown event:", ccevent.type)



#! /usr/bin/env python3
"""Unit tests for smartcard.CardConnection

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import sys
import unittest

sys.path += [".."]

try:
    from local_config import expectedATRinReader
except ImportError:
    print("execute test suite first to generate the local_config.py file")
    sys.exit()


from smartcard.CardConnection import CardConnection
from smartcard.Exceptions import CardConnectionException, NoCardException
from smartcard.scard import (
    SCARD_LEAVE_CARD,
    SCARD_RESET_CARD,
    SCARD_SHARE_EXCLUSIVE,
    SCARD_UNPOWER_CARD,
)
from smartcard.System import readers

SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]


class testcase_CardConnection(unittest.TestCase):
    """Test case for CardConnection."""

    def testcase_CardConnection(self):
        """Test with default protocols the response to SELECT DF_TELECOM."""
        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect()
                response, sw1, sw2 = cc.transmit(SELECT + DF_TELECOM)
                expectedSWs = {"9f 1a": 1, "6e 0": 2, "9f 20": 3, "9f 22": 4}
                self.assertEqual([], response)
                self.assertTrue(f"{sw1:x} {sw2:x}" in expectedSWs or "9f" == f"{sw1:x}")
            else:
                self.assertRaises(NoCardException, cc.connect)
        cc.disconnect()
        cc.release()

    def testcase_CardConnectionT0(self):
        """Test with T0 the response to SELECT DF_TELECOM."""

        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect(CardConnection.T0_protocol)
                response, sw1, sw2 = cc.transmit(SELECT + DF_TELECOM)
                expectedSWs = {"9f 1a": 1, "6e 0": 2, "9f 20": 3, "9f 22": 4}
                self.assertEqual([], response)
                self.assertTrue(f"{sw1:x} {sw2:x}" in expectedSWs or "9f" == f"{sw1:x}")
            else:
                self.assertRaises(NoCardException, cc.connect)
        cc.disconnect()
        cc.release()

    def testcase_CardConnectionT1inConnect(self):
        """Test that connecting with T1 on a T0 card fails."""

        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                # should fail since the test card does not support T1
                self.assertRaises(
                    CardConnectionException, cc.connect, CardConnection.T1_protocol
                )
            else:
                self.assertRaises(NoCardException, cc.connect)
        cc.disconnect()
        cc.release()

    def testcase_CardConnectionT1inTransmit(self):
        """Test that T1 in transmit for a T0 card fails."""

        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect()
                self.assertRaises(
                    CardConnectionException,
                    cc.transmit,
                    SELECT + DF_TELECOM,
                    CardConnection.T1_protocol,
                )
            else:
                self.assertRaises(NoCardException, cc.connect)
        cc.disconnect()
        cc.release()

    def testcase_CardConnectionT0T1(self):
        """Test test with T0 | T1 the response to SELECT DF_TELECOM."""

        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect(CardConnection.T0_protocol | CardConnection.T1_protocol)
                response, sw1, sw2 = cc.transmit(SELECT + DF_TELECOM)
                expectedSWs = {"9f 1a": 1, "6e 0": 2, "9f 20": 3, "9f 22": 4}
                self.assertEqual([], response)
                self.assertTrue(f"{sw1:x} {sw2:x}" in expectedSWs or "9f" == f"{sw1:x}")
            else:
                self.assertRaises(NoCardException, cc.connect)
        cc.disconnect()
        cc.release()

    def testcase_CardConnectionT0inTransmit(self):
        """Test with T0 in transmit the response to SELECT DF_TELECOM."""

        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect(CardConnection.T0_protocol)
                response, sw1, sw2 = cc.transmit(
                    SELECT + DF_TELECOM, CardConnection.T0_protocol
                )
                expectedSWs = {"9f 1a": 1, "6e 0": 2, "9f 20": 3, "9f 22": 4}
                self.assertEqual([], response)
                self.assertTrue(f"{sw1:x} {sw2:x}" in expectedSWs or "9f" == f"{sw1:x}")
            else:
                self.assertRaises(NoCardException, cc.connect)
        cc.disconnect()
        cc.release()

    def testcase_CardConnectionT0T1inTransmitMustFail(self):
        """Test with bad parameter in transmit the response to SELECT
        DF_TELECOM."""

        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect(CardConnection.T0_protocol | CardConnection.T1_protocol)
                self.assertRaises(
                    CardConnectionException,
                    cc.transmit,
                    SELECT + DF_TELECOM,
                    CardConnection.T0_protocol | CardConnection.T1_protocol,
                )
            else:
                self.assertRaises(NoCardException, cc.connect)
        cc.disconnect()
        cc.release()

    def testcase_CardReconnectProtocol(self):
        """Test .reconnect()"""
        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect(CardConnection.T0_protocol | CardConnection.T1_protocol)

                #  reconnect in T=1 when a T=0 card (GPK 8K) shall fail
                self.assertRaises(
                    CardConnectionException,
                    cc.reconnect,
                    protocol=CardConnection.T1_protocol,
                )
                cc.disconnect()
                cc.release()
            else:
                self.assertRaises(NoCardException, cc.connect)

    def testcase_CardReconnectMode(self):
        """Test .reconnect()"""
        for reader in readers():
            cc = reader.createConnection()
            cc2 = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect(CardConnection.T0_protocol | CardConnection.T1_protocol)

                #  reconnect in exclusive mode should fail
                self.assertRaises(
                    CardConnectionException, cc2.reconnect, mode=SCARD_SHARE_EXCLUSIVE
                )
                cc.disconnect()
                cc.release()
            else:
                self.assertRaises(NoCardException, cc.connect)

    def testcase_CardReconnectDisposition(self):
        """Test .reconnect()"""
        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                cc.connect(CardConnection.T0_protocol | CardConnection.T1_protocol)
                cc.reconnect(disposition=SCARD_LEAVE_CARD)
                cc.reconnect(disposition=SCARD_RESET_CARD)
                cc.reconnect(disposition=SCARD_UNPOWER_CARD)
                cc.disconnect()
                cc.release()
            else:
                self.assertRaises(NoCardException, cc.connect)

    def testcase_CardReconnectNoConnect(self):
        """Test .reconnect()"""
        for reader in readers():
            cc = reader.createConnection()
            if expectedATRinReader[str(reader)]:
                #  reconnect without connect first shall fail
                self.assertRaises(CardConnectionException, cc.reconnect)
            else:
                self.assertRaises(NoCardException, cc.connect)


if __name__ == "__main__":
    unittest.main(verbosity=1)



import pytest

import smartcard.ulist


def test_demonstrate_ulist_cannot_be_instantiated():
    """ulist() cannot be instantiated without an argument."""

    with pytest.raises(TypeError, match="'NoneType' object is not iterable"):
        smartcard.ulist.ulist()


@pytest.mark.xfail(reason="order should be ['x', 1, 2] not [1, 2, 'x']", strict=True)
def test_demonstrate_order_is_not_respected():
    """Order is not strictly respected."""

    instance = smartcard.ulist.ulist([1, 2])
    new_instance = ["x"] + instance
    assert new_instance == ["x", 1, 2]


@pytest.mark.xfail(reason="not all list methods are overwritten", strict=True)
def test_demonstrate_non_unique_behavior():
    """ulist.extend() breaks uniqueness guarantees."""

    instance = smartcard.ulist.ulist([1, 2])
    instance.extend([1, 2, 3])
    assert instance == [1, 2]  # actually, it's [1, 2, 1, 2, 3]


def test_ulist_methods():
    """This broadly tests the ulist implementation.

    It can be removed when ulist is removed.
    """

    smartcard.ulist.ulist([])
    smartcard.ulist.ulist([1])
    smartcard.ulist.ulist([1, 1])
    instance = smartcard.ulist.ulist([1])
    original_id = id(instance)
    assert instance
    assert len(instance) == 1
    assert instance == [1]
    assert id(instance) == original_id

    instance.append(1)
    assert instance
    assert len(instance) == 1
    assert instance == [1]
    assert id(instance) == original_id

    instance.append(2)
    assert instance
    assert len(instance) == 2
    assert instance == [1, 2]
    assert id(instance) == original_id

    new_instance = instance + [3]
    assert new_instance is not instance
    assert new_instance == [1, 2, 3]
    assert instance == [1, 2]
    assert id(instance) == original_id

    new_instance = instance + [1, 2]
    assert new_instance is not instance
    assert new_instance == [1, 2]
    assert instance == [1, 2]
    assert id(instance) == original_id

    new_instance = [1, 2] + instance
    assert new_instance is not instance
    assert new_instance == [1, 2]
    assert instance == [1, 2]
    assert id(instance) == original_id

    instance += 2
    assert instance == [1, 2]
    assert id(instance) == original_id

    instance.insert(0, 0)
    assert len(instance) == 3
    assert instance == [0, 1, 2]

    instance.insert(10_000, 0)
    assert len(instance) == 3
    assert instance == [0, 1, 2]

    value = instance.pop()
    assert value == 2
    assert instance == [0, 1]
    value = instance.pop(0)
    assert value == 0
    assert instance == [1]
    instance.pop()

    instance.append(0)
    instance.remove(0)
    with pytest.raises(ValueError):
        instance.remove(0)

    my_pristine_list = [0, 1, 2, 0, 1, 2, 0, 1, 2]
    instance += my_pristine_list
    assert my_pristine_list == [0, 1, 2, 0, 1, 2, 0, 1, 2]



#! /usr/bin/env python3
"""Unit test suite for scard python module.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import sys
import unittest

# so that we can locate configcheck
sys.path += [".."]
import configcheck


def suite():
    """suite"""
    modules_to_test = (
        "testcase_getatr",
        "testcase_getattrib",
        "testcase_geterrormessage",
        "testcase_listcards",
        "testcase_locatecards",
        "testcase_readergroups",
        "testcase_returncodes",
        "testcase_transaction",
    )
    testsuite_scard = unittest.TestSuite()
    for module in map(__import__, modules_to_test):
        testsuite_scard.addTest(unittest.TestLoader().loadTestsFromModule(module))
    return testsuite_scard


if __name__ == "__main__":
    configcheck.checklocalconfig()
    unittest.main(defaultTest="suite", verbosity=1)



#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: List card attributes

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import struct
import sys

import smartcard.util
from smartcard.scard import *

attributes = {
    SCARD_ATTR_ATR_STRING: "SCARD_ATTR_ATR_STRING",
    SCARD_ATTR_CHANNEL_ID: "SCARD_ATTR_CHANNEL_ID",
    SCARD_ATTR_CHARACTERISTICS: "SCARD_ATTR_CHARACTERISTICS",
    SCARD_ATTR_CURRENT_BWT: "SCARD_ATTR_CURRENT_BWT",
    SCARD_ATTR_CURRENT_CLK: "SCARD_ATTR_CURRENT_CLK",
    SCARD_ATTR_CURRENT_CWT: "SCARD_ATTR_CURRENT_CWT",
    SCARD_ATTR_CURRENT_D: "SCARD_ATTR_CURRENT_D",
    SCARD_ATTR_CURRENT_EBC_ENCODING: "SCARD_ATTR_CURRENT_EBC_ENCODING",
    SCARD_ATTR_CURRENT_F: "SCARD_ATTR_CURRENT_F",
    SCARD_ATTR_CURRENT_IFSC: "SCARD_ATTR_CURRENT_IFSC",
    SCARD_ATTR_CURRENT_IFSD: "SCARD_ATTR_CURRENT_IFSD",
    SCARD_ATTR_CURRENT_IO_STATE: "SCARD_ATTR_CURRENT_IO_STATE",
    SCARD_ATTR_CURRENT_N: "SCARD_ATTR_CURRENT_N",
    SCARD_ATTR_CURRENT_PROTOCOL_TYPE: "SCARD_ATTR_CURRENT_PROTOCOL_TYPE",
    SCARD_ATTR_CURRENT_W: "SCARD_ATTR_CURRENT_W",
    SCARD_ATTR_DEFAULT_CLK: "SCARD_ATTR_DEFAULT_CLK",
    SCARD_ATTR_DEFAULT_DATA_RATE: "SCARD_ATTR_DEFAULT_DATA_RATE",
    SCARD_ATTR_DEVICE_FRIENDLY_NAME_A: "SCARD_ATTR_DEVICE_FRIENDLY_NAME_A",
    SCARD_ATTR_DEVICE_FRIENDLY_NAME_W: "SCARD_ATTR_DEVICE_FRIENDLY_NAME_W",
    SCARD_ATTR_DEVICE_IN_USE: "SCARD_ATTR_DEVICE_IN_USE",
    SCARD_ATTR_DEVICE_SYSTEM_NAME_A: "SCARD_ATTR_DEVICE_SYSTEM_NAME_A",
    SCARD_ATTR_DEVICE_SYSTEM_NAME_W: "SCARD_ATTR_DEVICE_SYSTEM_NAME_W",
    SCARD_ATTR_DEVICE_UNIT: "SCARD_ATTR_DEVICE_UNIT",
    SCARD_ATTR_ESC_AUTHREQUEST: "SCARD_ATTR_ESC_AUTHREQUEST",
    SCARD_ATTR_ESC_CANCEL: "SCARD_ATTR_ESC_CANCEL",
    SCARD_ATTR_ESC_RESET: "SCARD_ATTR_ESC_RESET",
    SCARD_ATTR_EXTENDED_BWT: "SCARD_ATTR_EXTENDED_BWT",
    SCARD_ATTR_ICC_INTERFACE_STATUS: "SCARD_ATTR_ICC_INTERFACE_STATUS",
    SCARD_ATTR_ICC_PRESENCE: "SCARD_ATTR_ICC_PRESENCE",
    SCARD_ATTR_ICC_TYPE_PER_ATR: "SCARD_ATTR_ICC_TYPE_PER_ATR",
    SCARD_ATTR_MAXINPUT: "SCARD_ATTR_MAXINPUT",
    SCARD_ATTR_MAX_CLK: "SCARD_ATTR_MAX_CLK",
    SCARD_ATTR_MAX_DATA_RATE: "SCARD_ATTR_MAX_DATA_RATE",
    SCARD_ATTR_MAX_IFSD: "SCARD_ATTR_MAX_IFSD",
    SCARD_ATTR_POWER_MGMT_SUPPORT: "SCARD_ATTR_POWER_MGMT_SUPPORT",
    SCARD_ATTR_SUPRESS_T1_IFS_REQUEST: "SCARD_ATTR_SUPRESS_T1_IFS_REQUEST",
    SCARD_ATTR_USER_AUTH_INPUT_DEVICE: "SCARD_ATTR_USER_AUTH_INPUT_DEVICE",
    SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE: "SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE",
    SCARD_ATTR_VENDOR_IFD_SERIAL_NO: "SCARD_ATTR_VENDOR_IFD_SERIAL_NO",
    SCARD_ATTR_VENDOR_IFD_TYPE: "SCARD_ATTR_VENDOR_IFD_TYPE",
    SCARD_ATTR_VENDOR_IFD_VERSION: "SCARD_ATTR_VENDOR_IFD_VERSION",
    SCARD_ATTR_VENDOR_NAME: "SCARD_ATTR_VENDOR_NAME",
}
if "pcsclite" == resourceManager:
    extra_attributes = {
        SCARD_ATTR_ASYNC_PROTOCOL_TYPES: "SCARD_ATTR_ASYNC_PROTOCOL_TYPES",
        SCARD_ATTR_SYNC_PROTOCOL_TYPES: "SCARD_ATTR_SYNC_PROTOCOL_TYPES",
    }
    attributes.update(extra_attributes)


def printAttribute(attrib, value):
    print("-----------------", attributes[attrib], "-----------------")
    print(value)
    print(smartcard.util.toHexString(value, smartcard.util.HEX))
    print(struct.pack(*["<" + "B" * len(value)] + value))


if __name__ == "__main__":
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    if hresult != SCARD_S_SUCCESS:
        raise error("Failed to establish context: " + SCardGetErrorMessage(hresult))
    print("Context established!")

    try:
        hresult, readers = SCardListReaders(hcontext, [])
        if hresult != SCARD_S_SUCCESS:
            raise error("Failed to list readers: " + SCardGetErrorMessage(hresult))
        print("PCSC Readers:", readers)

        if len(readers) < 1:
            raise error("No smart card readers")

        for reader in readers:
            print("Trying to retrieve attributes of", reader)
            hresult, hcard, dwActiveProtocol = SCardConnect(
                hcontext,
                reader,
                SCARD_SHARE_SHARED,
                SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1,
            )
            if hresult != SCARD_S_SUCCESS:
                print(error, "Unable to connect: " + SCardGetErrorMessage(hresult))
            else:

                print("Connected with active protocol", dwActiveProtocol)

                try:
                    for i in list(attributes.keys()):
                        hresult, attrib = SCardGetAttrib(hcard, i)
                        if hresult == SCARD_S_SUCCESS:
                            printAttribute(i, attrib)
                        else:
                            print(
                                "-----------------", attributes[i], "-----------------"
                            )
                            print("error:", SCardGetErrorMessage(hresult))

                finally:
                    hresult = SCardDisconnect(hcard, SCARD_UNPOWER_CARD)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            "Failed to disconnect: " + SCardGetErrorMessage(hresult)
                        )
                    print("Disconnected")

    finally:
        hresult = SCardReleaseContext(hcontext)
        if hresult != SCARD_S_SUCCESS:
            raise error("Failed to release context: " + SCardGetErrorMessage(hresult))
        print("Released context.")

if "win32" == sys.platform:
    print("press Enter to continue")
    sys.stdin.read(1)
